@@ -1,12 +1,15 @@
 from .client_cmd import client_cmd  # noqa: F401
 from .combiner_cmd import combiner_cmd  # noqa: F401
 from .config_cmd import config_cmd  # noqa: F401
+from .controller_cmd import controller_cmd  # noqa: F401
+from .hooks_cmd import hooks_cmd  # noqa: F401
+from .login_cmd import login_cmd  # noqa: F401
 from .main import main  # noqa: F401
 from .model_cmd import model_cmd  # noqa: F401
 from .package_cmd import package_cmd  # noqa: F401
+from .project_cmd import project_cmd  # noqa: F401
 from .round_cmd import round_cmd  # noqa: F401
 from .run_cmd import run_cmd  # noqa: F401
 from .session_cmd import session_cmd  # noqa: F401
 from .status_cmd import status_cmd  # noqa: F401
 from .validation_cmd import validation_cmd  # noqa: F401
-from .controller_cmd import controller_cmd  # noqa: F401
