@@ -1,6 +1,6 @@
 import click
 
-from .main import main
+from fedn.cli.main import main
 
 
 @main.group("controller")
@@ -13,6 +13,6 @@ def controller_cmd(ctx):
 @controller_cmd.command("start")
 @click.pass_context
 def controller_cmd(ctx):
-    from fedn.network.api.server import start_server_api
+    from fedn.network.api.server import start_server_api  # noqa: PLC0415
 
     start_server_api()
