@@ -1,7 +1,11 @@
+import inspect
 import os
+import uuid
 
 import requests
 
+from fedn.network.combiner.hooks.serverfunctionsbase import ServerFunctionsBase
+
 __all__ = ["APIClient"]
 
 
@@ -18,7 +22,9 @@ class APIClient:
     :type verify: bool
     """
 
-    def __init__(self, host, port=None, secure=False, verify=False, token=None, auth_scheme=None):
+    def __init__(self, host: str, port: int = None, secure: bool = False, verify: bool = False, token: str = None, auth_scheme: str = None):
+        if "://" in host:
+            host = host.split("://")[1]
         self.host = host
         self.port = port
         self.secure = secure
@@ -33,6 +39,9 @@ class APIClient:
             token = os.environ.get("FEDN_AUTH_TOKEN", False)
 
         if token:
+            # Split the token if it contains a space (scheme + token).
+            if " " in token:
+                token = token.split()[1]
             self.headers = {"Authorization": f"{auth_scheme} {token}"}
 
     def _get_url(self, endpoint):
@@ -105,7 +114,7 @@ class APIClient:
         """
         _params = {"checksum": "true" if checksum else "false"}
 
-        response = requests.get(self._get_url("get_client_config"), params=_params, verify=self.verify, headers=self.headers)
+        response = requests.get(self._get_url_api_v1("clients/config"), params=_params, verify=self.verify, headers=self.headers)
 
         _json = response.json()
 
@@ -260,6 +269,8 @@ class APIClient:
         """
         _headers = self.headers.copy()
         _headers["X-Limit"] = "1"
+        _headers["X-Sort-Key"] = "committed_at"
+        _headers["X-Sort-Order"] = "desc"
 
         response = requests.get(self._get_url_api_v1("models/"), verify=self.verify, headers=_headers)
         _json = response.json()
@@ -281,14 +292,15 @@ class APIClient:
         """
         if not id:
             model = self.get_active_model()
-            if "id" in model:
-                id = model["id"]
+            if "model_id" in model:
+                id = model["model_id"]
             else:
                 return model
 
         _headers = self.headers.copy()
 
         _count: int = n_max if n_max else self.get_models_count()
+
         _headers["X-Limit"] = str(_count)
         _headers["X-Reverse"] = "true" if reverse else "false"
 
@@ -326,8 +338,16 @@ class APIClient:
         :return: A dict with success or failure message.
         :rtype: dict
         """
+        if path.endswith(".npz"):
+            helper = "numpyhelper"
+        elif path.endswith(".bin"):
+            helper = "binaryhelper"
+
+        if helper:
+            response = requests.put(self._get_url_api_v1("helpers/active"), json={"helper": helper}, verify=self.verify, headers=self.headers)
+
         with open(path, "rb") as file:
-            response = requests.post(self._get_url("set_initial_model"), files={"file": file}, verify=self.verify, headers=self.headers)
+            response = requests.post(self._get_url_api_v1("models/"), files={"file": file}, data={"helper": helper}, verify=self.verify, headers=self.headers)
         return response.json()
 
     # --- Packages --- #
@@ -395,7 +415,7 @@ class APIClient:
         :return: The checksum.
         :rtype: dict
         """
-        response = requests.get(self._get_url("get_package_checksum"), verify=self.verify, headers=self.headers)
+        response = requests.get(self._get_url_api_v1("packages/checksum"), verify=self.verify, headers=self.headers)
 
         _json = response.json()
 
@@ -409,7 +429,7 @@ class APIClient:
         :return: Message with success or failure.
         :rtype: dict
         """
-        response = requests.get(self._get_url("download_package"), verify=self.verify, headers=self.headers)
+        response = requests.get(self._get_url_api_v1("packages/download"), verify=self.verify, headers=self.headers)
         if response.status_code == 200:
             with open(path, "wb") as file:
                 file.write(response.content)
@@ -417,7 +437,7 @@ class APIClient:
         else:
             return {"success": False, "message": "Failed to download package."}
 
-    def set_active_package(self, path: str, helper: str, name: str = None, description: str = None):
+    def set_active_package(self, path: str, helper: str, name: str, description: str = ""):
         """Set the compute package in the statestore.
 
         :param path: The file path of the compute package to set.
@@ -429,7 +449,7 @@ class APIClient:
         """
         with open(path, "rb") as file:
             response = requests.post(
-                self._get_url("set_package"),
+                self._get_url_api_v1("packages/"),
                 files={"file": file},
                 data={"helper": helper, "name": name, "description": description},
                 verify=self.verify,
@@ -489,25 +509,38 @@ class APIClient:
 
     # --- Sessions --- #
 
-    def get_session(self, id: str):
+    def get_session(self, id: str = None, name: str = None):
         """Get a session from the statestore.
 
         :param id: The session id to get.
         :type id: str
+        :param name: The session name to get.
+        :type name: str
         :return: Session.
         :rtype: dict
         """
-        response = requests.get(self._get_url_api_v1(f"sessions/{id}"), self.verify, headers=self.headers)
-
-        _json = response.json()
+        if name:
+            response = requests.get(self._get_url_api_v1(f"sessions?name={name}"), verify=self.verify, headers=self.headers)
+            _json = response.json()
+            if "result" in _json and len(_json["result"]) > 0:
+                _json = _json["result"][0]
+            else:
+                _json = {"message": "Session not found."}
+        elif id:
+            response = requests.get(self._get_url_api_v1(f"sessions/{id}"), verify=self.verify, headers=self.headers)
+            _json = response.json()
+        else:
+            _json = {"message": "No id or name provided."}
 
         return _json
 
-    def get_sessions(self, n_max: int = None):
+    def get_sessions(self, n_max: int = None, name: str = None):
         """Get sessions from the statestore.
 
         :param n_max: The maximum number of sessions to get (If none all will be fetched).
         :type n_max: int
+        :param name: The session name to get.
+        :type name: str
         :return: Sessions.
         :rtype: dict
         """
@@ -516,7 +549,11 @@ class APIClient:
         if n_max:
             _headers["X-Limit"] = str(n_max)
 
-        response = requests.get(self._get_url_api_v1("sessions/"), verify=self.verify, headers=_headers)
+        url = self._get_url_api_v1("sessions/")
+        if name:
+            url += f"?name={name}"
+
+        response = requests.get(url, verify=self.verify, headers=_headers)
 
         _json = response.json()
 
@@ -562,7 +599,7 @@ class APIClient:
 
     def start_session(
         self,
-        id: str = None,
+        name: str = None,
         aggregator: str = "fedavg",
         aggregator_kwargs: dict = None,
         model_id: str = None,
@@ -571,14 +608,15 @@ class APIClient:
         round_buffer_size: int = -1,
         delete_models: bool = True,
         validate: bool = True,
-        helper: str = "",
+        helper: str = None,
         min_clients: int = 1,
         requested_clients: int = 8,
+        server_functions: ServerFunctionsBase = None,
     ):
         """Start a new session.
 
-        :param id: The session id to start.
-        :type id: str
+        :param name: The name of the session
+        :type name: str
         :param aggregator: The aggregator plugin to use.
         :type aggregator: str
         :param model_id: The id of the initial model.
@@ -602,21 +640,189 @@ class APIClient:
         :return: A dict with success or failure message and session config.
         :rtype: dict
         """
+        is_splitlearning = aggregator == "splitlearningagg"
+        if is_splitlearning:
+            return self.start_splitlearning_session(
+                name, model_id, round_timeout, rounds, round_buffer_size, delete_models, validate, min_clients, requested_clients
+            )
+
+        if model_id is None:
+            _headers = self.headers.copy()
+            _headers["X-Limit"] = "1"
+            _headers["X-Sort-Key"] = "committed_at"
+            _headers["X-Sort-Order"] = "desc"
+            response = requests.get(self._get_url_api_v1("models/"), verify=self.verify, headers=_headers)
+            if response.status_code == 200:
+                json = response.json()
+                if "result" in json and len(json["result"]) > 0:
+                    model_id = json["result"][0]["model_id"]
+                else:
+                    return {"message": "No models found in the repository"}
+            else:
+                return {"message": "No models found in the repository"}
+
+        if helper is None:
+            response = requests.get(self._get_url_api_v1("helpers/active"), verify=self.verify, headers=self.headers)
+            if response.status_code == 400:
+                helper = "numpyhelper"
+            elif response.status_code == 200:
+                helper = response.json()
+            else:
+                return {"message": "An unexpected error occurred when getting the active helper"}
+
         response = requests.post(
-            self._get_url("start_session"),
+            self._get_url_api_v1("sessions/"),
             json={
-                "session_id": id,
-                "aggregator": aggregator,
-                "aggregator_kwargs": aggregator_kwargs,
-                "model_id": model_id,
-                "round_timeout": round_timeout,
+                "name": name,
+                "seed_model_id": model_id,
+                "session_config": {
+                    "aggregator": aggregator,
+                    "aggregator_kwargs": aggregator_kwargs,
+                    "round_timeout": round_timeout,
+                    "buffer_size": round_buffer_size,
+                    "model_id": model_id,
+                    "delete_models_storage": delete_models,
+                    "clients_required": min_clients,
+                    "requested_clients": requested_clients,
+                    "validate": validate,
+                    "helper_type": helper,
+                    "server_functions": None if server_functions is None else inspect.getsource(server_functions),
+                },
+            },
+            verify=self.verify,
+            headers=self.headers,
+        )
+
+        if response.status_code == 201:
+            session_id = response.json()["session_id"]
+            response = requests.post(
+                self._get_url_api_v1("sessions/start"),
+                json={
+                    "session_id": session_id,
+                    "rounds": rounds,
+                    "round_timeout": round_timeout,
+                },
+                verify=self.verify,
+                headers=self.headers,
+            )
+            # Try to parse JSON, but handle the case where it fails
+            try:
+                response_json = response.json()
+                response_json["session_id"] = session_id
+                return response_json
+            except requests.exceptions.JSONDecodeError:
+                # Handle invalid JSON response
+                return {"success": response.status_code < 400, "session_id": session_id, "message": f"Session started with status code {response.status_code}"}
+
+        _json = response.json()
+        return _json
+
+    def start_splitlearning_session(
+        self,
+        name: str = None,
+        model_id: str = None,
+        round_timeout: int = 180,
+        rounds: int = 5,
+        round_buffer_size: int = -1,
+        delete_models: bool = True,
+        validate: bool = False,
+        min_clients: int = 1,
+        requested_clients: int = 8,
+    ):
+        """Start a new splitlearning session.
+
+        :param name: The name of the session
+        :type name: str
+        :param model_id: The id of the initial model.
+        :type model_id: str
+        :param round_timeout: The round timeout to use in seconds.
+        :type round_timeout: int
+        :param rounds: The number of rounds to perform.
+        :type rounds: int
+        :param round_buffer_size: The round buffer size to use.
+        :type round_buffer_size: int
+        :param delete_models: Whether to delete models after each round at combiner (save storage).
+        :type delete_models: bool
+        :param validate: Whether to validate the model after each round.
+        :type validate: bool
+        :param min_clients: The minimum number of clients required.
+        :type min_clients: int
+        :param requested_clients: The requested number of clients.
+        :type requested_clients: int
+        :return: A dict with success or failure message and session config.
+        :rtype: dict
+        """
+        response = requests.post(
+            self._get_url_api_v1("sessions/"),
+            json={
+                "name": name,
+                "session_config": {
+                    "aggregator": "splitlearningagg",
+                    "model_id": model_id,
+                    "round_timeout": round_timeout,
+                    "buffer_size": round_buffer_size,
+                    "delete_models_storage": delete_models,
+                    "clients_required": min_clients,
+                    "requested_clients": requested_clients,
+                    "validate": validate,
+                    "helper_type": "splitlearninghelper",
+                },
+            },
+            verify=self.verify,
+            headers=self.headers,
+        )
+
+        if response.status_code == 201:
+            session_id = response.json()["session_id"]
+            response = requests.post(
+                self._get_url_api_v1("sessions/start_splitlearning_session"),
+                json={
+                    "session_id": session_id,
+                    "rounds": rounds,
+                    "round_timeout": round_timeout,
+                },
+                verify=self.verify,
+                headers=self.headers,
+            )
+            response_json = response.json()
+            response_json["session_id"] = session_id
+            return response_json
+
+        _json = response.json()
+        return _json
+
+    def continue_session(self, session_id: str, rounds: int = 5, round_timeout: int = 180):
+        """Continue a session.
+
+        :param session_id: The id of the session to continue.
+        :type session_id: str
+        :param rounds: The number of rounds to perform.
+        :type rounds: int
+        :param round_timeout: The round timeout to use in seconds.
+        :type round_timeout: int
+        :return: A dict with success or failure message and session config.
+        :rtype: dict
+        """
+        if not session_id:
+            return {"message": "No session id provided."}
+        if rounds is None or rounds <= 0:
+            return {"message": "Invalid number of rounds provided. Must be greater than 0."}
+        if round_timeout is None or round_timeout <= 0:
+            return {"message": "Invalid round timeout provided. Must be greater than 0."}
+        # Check if session exists
+        session = self.get_session(session_id)
+        if not session or "session_id" not in session:
+            return {"message": "Session not found."}
+        # Check if session is finished
+        if not self.session_is_finished(session_id):
+            return {"message": "Session is already running."}
+
+        response = requests.post(
+            self._get_url_api_v1("sessions/start"),
+            json={
+                "session_id": session_id,
                 "rounds": rounds,
-                "round_buffer_size": round_buffer_size,
-                "delete_models": delete_models,
-                "validate": validate,
-                "helper": helper,
-                "min_clients": min_clients,
-                "requested_clients": requested_clients,
+                "round_timeout": round_timeout,
             },
             verify=self.verify,
             headers=self.headers,
@@ -787,3 +993,128 @@ class APIClient:
         _json = response.json()
 
         return _json
+
+    # --- Predictions --- #
+
+    def get_predictions(
+        self,
+        model_id: str = None,
+        correlation_id: str = None,
+        sender_name: str = None,
+        sender_role: str = None,
+        receiver_name: str = None,
+        receiver_role: str = None,
+        n_max: int = None,
+    ):
+        """Get predictions from the statestore. Filter by input parameters.
+
+        :param model_id: The model id to get predictions for.
+        :type model_id: str
+        :param correlation_id: The correlation id to get predictions for.
+        :type correlation_id: str
+        :param sender_name: The sender name to get predictions for.
+        :type sender_name: str
+        :param sender_role: The sender role to get predictions for.
+        :type sender_role: str
+        :param receiver_name: The receiver name to get predictions for.
+        :type receiver_name: str
+        :param receiver_role: The receiver role to get predictions for.
+        :type receiver_role: str
+        :param n_max: The maximum number of predictions to get (If none all will be fetched).
+        :type n_max: int
+        :return: Predictions.
+        :rtype: dict
+        """
+        _params = {}
+
+        if model_id:
+            _params["modelId"] = model_id
+
+        if correlation_id:
+            _params["correlationId"] = correlation_id
+
+        if sender_name:
+            _params["sender.name"] = sender_name
+
+        if sender_role:
+            _params["sender.role"] = sender_role
+
+        if receiver_name:
+            _params["receiver.name"] = receiver_name
+
+        if receiver_role:
+            _params["receiver.role"] = receiver_role
+
+        _headers = self.headers.copy()
+
+        if n_max:
+            _headers["X-Limit"] = str(n_max)
+
+        response = requests.get(self._get_url_api_v1("predictions/"), params=_params, verify=self.verify, headers=_headers)
+
+        _json = response.json()
+
+        return _json
+
+    def start_predictions(self, prediction_id: str = None, model_id: str = None):
+        """Start predictions for a model.
+
+        :param model_id: The model id to start predictions for.
+        :type model_id: str
+        :param data: The data to predict.
+        :type data: dict
+        :return: A dict with success or failure message.
+        :rtype: dict
+        """
+        if not prediction_id:
+            prediction_id = str(uuid.uuid4())
+        response = requests.post(
+            self._get_url_api_v1("predictions/start"),
+            json={"prediction_id": prediction_id, "model_id": model_id},
+            verify=self.verify,
+            headers=self.headers,
+        )
+
+        _json = response.json()
+
+        return _json
+
+    # --- Client --- #
+
+    def get_current_attributes(self, client_list):
+        """Get the current attributes of the client.
+
+        :param client_list: The list of clients to get the attributes for or a single client_id
+        :type client_list: list|str
+        :raises ValueError: If client_list is not a list or empty.
+        :return: The current attributes of the client.
+        :rtype: dict
+        """
+        if not isinstance(client_list, list):
+            if isinstance(client_list, str):
+                client_list = [client_list]
+            else:
+                raise ValueError("client_list must be a list")
+        if len(client_list) == 0:
+            raise ValueError("client_list must not be empty")
+        json = {"client_ids": client_list}
+        response = requests.post(self._get_url_api_v1("attributes/current"), json=json, verify=self.verify, headers=self.headers)
+        _json = response.json()
+        return _json
+
+    def add_attributes(self, attribute: dict) -> dict:
+        """Add or update client attributes via the controller API.
+
+        :param attribute: A dict matching AttributeDTO.schema, e.g.
+            {
+                "key": "charging",
+                "value": "true",
+                "sender": {"name": "", "role": "", "client_id": "abc123"}
+            }
+        :return: Parsed JSON response from the server.
+        :rtype: dict
+        """
+        url = self._get_url_api_v1("attributes/")
+        response = requests.post(url, json=attribute, headers=self.headers, verify=self.verify)
+        response.raise_for_status()
+        return response.json()
