@@ -1,8 +1,10 @@
-import base64
+import os
+from typing import List
 
 from fedn.common.log_config import logger
 from fedn.network.combiner.interfaces import CombinerInterface
 from fedn.network.loadbalancer.leastpacked import LeastPacked
+from fedn.network.storage.dbconnection import DatabaseConnection
 
 __all__ = ("Network",)
 
@@ -13,11 +15,11 @@ class Network:
     Some methods has been moved to :class:`fedn.network.api.interface.API`.
     """
 
-    def __init__(self, control, statestore, load_balancer=None):
+    def __init__(self, control, network_id: str, dbconn: DatabaseConnection, load_balancer=None):
         """ """
-        self.statestore = statestore
         self.control = control
-        self.id = statestore.network_id
+        self.id = network_id
+        self.db = dbconn
 
         if not load_balancer:
             self.load_balancer = LeastPacked(self)
@@ -38,55 +40,33 @@ class Network:
                 return combiner
         return None
 
-    def get_combiners(self):
+    def get_combiners(self) -> List[CombinerInterface]:
         """Get all combiners in the network.
 
         :return: list of combiners objects
         :rtype: list(:class:`fedn.network.combiner.interfaces.CombinerInterface`)
         """
-        data = self.statestore.get_combiners()
+        result = self.db.combiner_store.list(limit=0, skip=0, sort_key=None)
         combiners = []
-        for c in data["result"]:
-            if c["certificate"]:
-                cert = base64.b64decode(c["certificate"])
-                key = base64.b64decode(c["key"])
+        for combiner in result:
+            name = combiner.name.upper()
+            # General certificate handling, same for all combiners.
+            if os.environ.get("FEDN_GRPC_CERT_PATH"):
+                with open(os.environ.get("FEDN_GRPC_CERT_PATH"), "rb") as f:
+                    cert = f.read()
+            # Specific certificate handling for each combiner.
+            elif os.environ.get(f"FEDN_GRPC_CERT_PATH_{name}"):
+                cert_path = os.environ.get(f"FEDN_GRPC_CERT_PATH_{name}")
+                with open(cert_path, "rb") as f:
+                    cert = f.read()
             else:
                 cert = None
-                key = None
-
-            combiners.append(CombinerInterface(c["parent"], c["name"], c["address"], c["fqdn"], c["port"], certificate=cert, key=key, ip=c["ip"]))
+            combiners.append(
+                CombinerInterface(combiner.parent, combiner.name, combiner.address, combiner.fqdn, combiner.port, certificate=cert, ip=combiner.ip)
+            )
 
         return combiners
 
-    def add_combiner(self, combiner):
-        """Add a new combiner to the network.
-
-        :param combiner: The combiner instance object
-        :type combiner: :class:`fedn.network.combiner.interfaces.CombinerInterface`
-        :return: None
-        """
-        if not self.control.idle():
-            logger.warning("Reducer is not idle, cannot add additional combiner.")
-            return
-
-        if self.get_combiner(combiner.name):
-            return
-
-        logger.info("adding combiner {}".format(combiner.name))
-        self.statestore.set_combiner(combiner.to_dict())
-
-    def remove_combiner(self, combiner):
-        """Remove a combiner from the network.
-
-        :param combiner: The combiner instance object
-        :type combiner: :class:`fedn.network.combiner.interfaces.CombinerInterface`
-        :return: None
-        """
-        if not self.control.idle():
-            logger.warning("Reducer is not idle, cannot remove combiner.")
-            return
-        self.statestore.delete_combiner(combiner.name)
-
     def find_available_combiner(self):
         """Find an available combiner in the network.
 
@@ -105,48 +85,3 @@ class Network:
         """
         # TODO: Implement strategy to handle an unavailable combiner.
         logger.warning("REDUCER CONTROL: Combiner {} unavailable.".format(combiner.name))
-
-    def add_client(self, client):
-        """Add a new client to the network.
-
-        :param client: The client instance object
-        :type client: dict
-        :return: None
-        """
-        if self.get_client(client["client_id"]):
-            return
-
-        logger.info("adding client {}".format(client["client_id"]))
-        self.statestore.set_client(client)
-
-    def get_client(self, name):
-        """Get client by name.
-
-        :param name: name of client
-        :type name: str
-        :return: The client instance object
-        :rtype: ObjectId
-        """
-        ret = self.statestore.get_client(name)
-        return ret
-
-    def update_client_data(self, client_data, status, role):
-        """Update client status in statestore.
-
-        :param client_data: The client instance object
-        :type client_data: dict
-        :param status: The client status
-        :type status: str
-        :param role: The client role
-        :type role: str
-        :return: None
-        """
-        self.statestore.update_client_status(client_data, status, role)
-
-    def get_client_info(self):
-        """List available client in statestore.
-
-        :return: list of client objects
-        :rtype: list(ObjectId)
-        """
-        return self.statestore.list_clients()
