@@ -1,14 +1,12 @@
 from flask import Blueprint, jsonify, request
 
+from fedn.common.log_config import logger
 from fedn.network.api.auth import jwt_auth_required
-from fedn.network.api.v1.shared import api_version, get_post_data_to_kwargs, get_typed_list_headers, mdb
-from fedn.network.storage.statestore.stores.combiner_store import CombinerStore
-from fedn.network.storage.statestore.stores.shared import EntityNotFound
+from fedn.network.api.v1.shared import api_version, get_post_data_to_kwargs, get_typed_list_headers
+from fedn.network.controller.control import Control
 
 bp = Blueprint("combiner", __name__, url_prefix=f"/api/{api_version}/combiners")
 
-combiner_store = CombinerStore(mdb, "network.combiners")
-
 
 @bp.route("/", methods=["GET"])
 @jwt_auth_required(role="admin")
@@ -102,18 +100,18 @@ def get_combiners():
                     type: string
     """
     try:
-        limit, skip, sort_key, sort_order, _ = get_typed_list_headers(request.headers)
+        db = Control.instance().db
+        limit, skip, sort_key, sort_order = get_typed_list_headers(request.headers)
 
         kwargs = request.args.to_dict()
 
-        combiners = combiner_store.list(limit, skip, sort_key, sort_order, use_typing=False, **kwargs)
-
-        result = combiners["result"]
-
-        response = {"count": combiners["count"], "result": result}
+        combiners = db.combiner_store.list(limit, skip, sort_key, sort_order, **kwargs)
+        count = db.combiner_store.count(**kwargs)
+        response = {"count": count, "result": [combiner.to_dict() for combiner in combiners]}
 
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -185,18 +183,18 @@ def list_combiners():
                     type: string
     """
     try:
-        limit, skip, sort_key, sort_order, _ = get_typed_list_headers(request.headers)
+        db = Control.instance().db
+        limit, skip, sort_key, sort_order = get_typed_list_headers(request.headers)
 
         kwargs = get_post_data_to_kwargs(request)
 
-        combiners = combiner_store.list(limit, skip, sort_key, sort_order, use_typing=False, **kwargs)
-
-        result = combiners["result"]
-
-        response = {"count": combiners["count"], "result": result}
+        combiners = db.combiner_store.list(limit, skip, sort_key, sort_order, **kwargs)
+        count = db.combiner_store.count(**kwargs)
+        response = {"count": count, "result": [combiner.to_dict() for combiner in combiners]}
 
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -239,11 +237,13 @@ def get_combiners_count():
                         type: string
     """
     try:
+        db = Control.instance().db
         kwargs = request.args.to_dict()
-        count = combiner_store.count(**kwargs)
+        count = db.combiner_store.count(**kwargs)
         response = count
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -288,11 +288,13 @@ def combiners_count():
                         type: string
     """
     try:
+        db = Control.instance().db
         kwargs = get_post_data_to_kwargs(request)
-        count = combiner_store.count(**kwargs)
+        count = db.combiner_store.count(**kwargs)
         response = count
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -331,11 +333,102 @@ def get_combiner(id: str):
                         type: string
     """
     try:
-        combiner = combiner_store.get(id, use_typing=False)
-        response = combiner
+        db = Control.instance().db
+        combiner = db.combiner_store.get(id)
+        if combiner is None:
+            return jsonify({"message": f"Entity with id: {id} not found"}), 404
+        return jsonify(combiner.to_dict()), 200
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
+        return jsonify({"message": "An unexpected error occurred"}), 500
 
-        return jsonify(response), 200
-    except EntityNotFound:
-        return jsonify({"message": f"Entity with id: {id} not found"}), 404
-    except Exception:
+
+@bp.route("/<string:id>", methods=["DELETE"])
+@jwt_auth_required(role="admin")
+def delete_combiner(id: str):
+    """Delete combiner
+    Deletes a combiner based on the provided id.
+    ---
+    tags:
+        - Combiners
+    parameters:
+      - name: id
+        in: path
+        required: true
+        type: string
+        description: The id of the combiner
+    responses:
+        200:
+            description: The combiner was deleted
+        404:
+            description: The combiner was not found
+            schema:
+                type: object
+                properties:
+                    error:
+                        type: string
+        500:
+            description: An error occurred
+            schema:
+                type: object
+                properties:
+                    message:
+                        type: string
+    """
+    try:
+        db = Control.instance().db
+        result: bool = db.combiner_store.delete(id)
+        if not result:
+            return jsonify({"message": f"Entity with id: {id} not found"}), 404
+        msg = "Combiner deleted" if result else "Combiner not deleted"
+        return jsonify({"message": msg}), 200
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
+        return jsonify({"message": "An unexpected error occurred"}), 500
+
+
+@bp.route("/clients/count", methods=["POST"])
+@jwt_auth_required(role="admin")
+def number_of_clients_connected():
+    """Number of clients connected
+    Retrieves the number of clients connected to the combiner.
+    ---
+    tags:
+        - Combiners
+    parameters:
+      - name: combiners
+        in: body
+        required: true
+        type: object
+        description: Object containing the ids of the combiners
+        schema:
+          type: object
+          properties:
+            combiners:
+                type: string
+    responses:
+        200:
+            description: A list of objects containing the number of clients connected to each combiner
+            schema:
+                type: Array
+        500:
+            description: An error occurred
+            schema:
+                type: object
+                properties:
+                    message:
+                        type: string
+    """
+    try:
+        db = Control.instance().db
+        data = request.get_json()
+        combiners = data.get("combiners", "")
+        combiners = combiners.split(",") if combiners else []
+        response = db.client_store.connected_client_count(combiners)
+
+        result = {"result": response}
+
+        return jsonify(result), 200
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
