@@ -1,24 +1,17 @@
 import io
+from io import BytesIO
 
 import numpy as np
 from flask import Blueprint, jsonify, request, send_file
 
+from fedn.common.log_config import logger
 from fedn.network.api.auth import jwt_auth_required
-from fedn.network.api.shared import modelstorage_config
-from fedn.network.api.v1.shared import api_version, get_limit, get_post_data_to_kwargs, get_reverse, get_typed_list_headers, mdb
-from fedn.network.storage.s3.base import RepositoryBase
-from fedn.network.storage.s3.miniorepository import MINIORepository
-from fedn.network.storage.statestore.stores.model_store import ModelStore
-from fedn.network.storage.statestore.stores.shared import EntityNotFound
+from fedn.network.api.v1.shared import api_version, get_limit, get_post_data_to_kwargs, get_reverse, get_typed_list_headers
+from fedn.network.controller.control import Control
+from fedn.network.storage.statestore.stores.shared import EntityNotFound, MissingFieldError, ValidationError
 
 bp = Blueprint("model", __name__, url_prefix=f"/api/{api_version}/models")
 
-model_store = ModelStore(mdb, "control.model")
-repository: RepositoryBase = None
-
-if modelstorage_config["storage_type"] == "S3":
-    repository = MINIORepository(modelstorage_config["storage_config"])
-
 
 @bp.route("/", methods=["GET"])
 @jwt_auth_required(role="admin")
@@ -107,17 +100,18 @@ def get_models():
                     type: string
     """
     try:
-        limit, skip, sort_key, sort_order, _ = get_typed_list_headers(request.headers)
+        db = Control.instance().db
+        limit, skip, sort_key, sort_order = get_typed_list_headers(request.headers)
         kwargs = request.args.to_dict()
 
-        models = model_store.list(limit, skip, sort_key, sort_order, use_typing=False, **kwargs)
-
-        result = models["result"]
-
-        response = {"count": models["count"], "result": result}
+        models = db.model_store.list(limit, skip, sort_key, sort_order, **kwargs)
+        result = [model.to_dict() for model in models]
+        count = db.model_store.count(**kwargs)
+        response = {"count": count, "result": result}
 
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -192,17 +186,18 @@ def list_models():
                     type: string
     """
     try:
-        limit, skip, sort_key, sort_order, _ = get_typed_list_headers(request.headers)
+        db = Control.instance().db
+        limit, skip, sort_key, sort_order = get_typed_list_headers(request.headers)
         kwargs = get_post_data_to_kwargs(request)
 
-        models = model_store.list(limit, skip, sort_key, sort_order, use_typing=False, **kwargs)
-
-        result = models["result"]
-
-        response = {"count": models["count"], "result": result}
+        models = db.model_store.list(limit, skip, sort_key, sort_order, **kwargs)
+        result = [model.to_dict() for model in models]
+        count = db.model_store.count(**kwargs)
+        response = {"count": count, "result": result}
 
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -246,11 +241,13 @@ def get_models_count():
                         type: string
     """
     try:
+        db = Control.instance().db
         kwargs = request.args.to_dict()
-        count = model_store.count(**kwargs)
+        count = db.model_store.count(**kwargs)
         response = count
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -298,11 +295,13 @@ def models_count():
                         type: string
     """
     try:
+        db = Control.instance().db
         kwargs = get_post_data_to_kwargs(request)
-        count = model_store.count(**kwargs)
+        count = db.model_store.count(**kwargs)
         response = count
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -341,14 +340,16 @@ def get_model(id: str):
                         type: string
     """
     try:
-        model = model_store.get(id, use_typing=False)
+        db = Control.instance().db
+        model = db.model_store.get(id)
 
-        response = model
+        if model is None:
+            return jsonify({"message": f"Entity with id: {id} not found"}), 404
 
+        response = model.to_dict()
         return jsonify(response), 200
-    except EntityNotFound:
-        return jsonify({"message": f"Entity with id: {id} not found"}), 404
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -392,28 +393,36 @@ def patch_model(id: str):
                         type: string
     """
     try:
-        model = model_store.get(id, use_typing=False)
+        db = Control.instance().db
+        existing_model = db.model_store.get(id)
+        if existing_model is None:
+            return jsonify({"message": f"Entity with id: {id} not found"}), 404
 
         data = request.get_json()
-        _id = model["id"]
 
-        # Update the model with the new data
-        # Only update the fields that are present in the request
-        for key, value in data.items():
-            if key in ["_id", "model"]:
-                continue
-            model[key] = value
+        data.pop("model", None)
+        data.pop("model_id", None)
 
-        success, message = model_store.update(_id, model)
+        existing_model.patch_with(data, throw_on_extra_keys=False)
+        updated_model = db.model_store.update(existing_model)
 
-        if success:
-            response = model
-            return jsonify(response), 200
+        response = updated_model.to_dict()
+        return jsonify(response), 200
 
-        return jsonify({"message": f"Failed to update model: {message}"}), 500
-    except EntityNotFound:
+    except EntityNotFound as e:
+        logger.error(f"Entity not found: {e}")
         return jsonify({"message": f"Entity with id: {id} not found"}), 404
-    except Exception:
+    except ValidationError as e:
+        logger.error(f"Validation error: {e}")
+        return jsonify({"message": e.user_message()}), 400
+    except MissingFieldError as e:
+        logger.error(f"Missing field error: {e}")
+        return jsonify({"message": e.user_message()}), 400
+    except ValueError as e:
+        logger.error(f"ValueError occured: {e}")
+        return jsonify({"message": "Invalid object"}), 400
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -457,20 +466,33 @@ def put_model(id: str):
                         type: string
     """
     try:
-        model = model_store.get(id, use_typing=False)
+        db = Control.instance().db
+        model = db.model_store.get(id)
+        if model is None:
+            return jsonify({"message": f"Entity with id: {id} not found"}), 404
         data = request.get_json()
-        _id = model["id"]
+        data.pop("model", None)
+        data["model_id"] = id
 
-        success, message = model_store.update(_id, data)
+        model.populate_with(data)
+        new_model = db.model_store.update(model)
+        response = new_model.to_dict()
+        return jsonify(response), 200
 
-        if success:
-            response = model
-            return jsonify(response), 200
-
-        return jsonify({"message": f"Failed to update model: {message}"}), 500
-    except EntityNotFound:
+    except EntityNotFound as e:
+        logger.error(f"Entity not found: {e}")
         return jsonify({"message": f"Entity with id: {id} not found"}), 404
-    except Exception:
+    except ValidationError as e:
+        logger.error(f"Validation error: {e}")
+        return jsonify({"message": e.user_message()}), 400
+    except MissingFieldError as e:
+        logger.error(f"Missing field error: {e}")
+        return jsonify({"message": e.user_message()}), 400
+    except ValueError as e:
+        logger.error(f"ValueError occured: {e}")
+        return jsonify({"message": "Invalid object"}), 400
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -515,16 +537,19 @@ def get_descendants(id: str):
                         type: string
     """
     try:
+        db = Control.instance().db
         limit = get_limit(request.headers)
 
-        descendants = model_store.list_descendants(id, limit or 10, use_typing=False)
+        descendants = db.model_store.list_descendants(id, limit or 10)
 
-        response = descendants
+        if descendants is None:
+            return jsonify({"message": f"Entity with id: {id} not found"}), 404
 
+        response = [model.to_dict() for model in descendants]
         return jsonify(response), 200
-    except EntityNotFound:
-        return jsonify({"message": f"Entity with id: {id} not found"}), 404
-    except Exception:
+
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -580,20 +605,55 @@ def get_ancestors(id: str):
                         type: string
     """
     try:
+        db = Control.instance().db
         limit = get_limit(request.headers)
         reverse = get_reverse(request.headers)
         include_self_param: str = request.args.get("include_self")
 
         include_self: bool = include_self_param and include_self_param.lower() == "true"
 
-        ancestors = model_store.list_ancestors(id, limit or 10, include_self=include_self, reverse=reverse, use_typing=False)
-
-        response = ancestors
-
+        ancestors = db.model_store.list_ancestors(id, limit or 10, include_self=include_self, reverse=reverse)
+        if ancestors is None:
+            return jsonify({"message": f"Entity with id: {id} not found"}), 404
+        response = [model.to_dict() for model in ancestors]
         return jsonify(response), 200
-    except EntityNotFound:
-        return jsonify({"message": f"Entity with id: {id} not found"}), 404
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
+        return jsonify({"message": "An unexpected error occurred"}), 500
+
+
+@bp.route("/leaf-nodes", methods=["GET"])
+@jwt_auth_required(role="admin")
+def get_leaf_nodes():
+    """Get model leaf nodes
+    Retrieves a list of
+    ---
+    tags:
+        - Models
+    responses:
+      200:
+        description: A list of models.
+        schema:
+            type: object
+            properties:
+                type: array
+                items:
+                    $ref: '#/definitions/Model'
+      500:
+        description: An error occurred
+        schema:
+            type: object
+            properties:
+                error:
+                    type: string
+    """
+    try:
+        db = Control.instance().db
+        leaf_nodes = db.model_store.get_leaf_nodes()
+        response = [model.to_dict() for model in leaf_nodes]
+        return jsonify(response), 200
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -631,18 +691,20 @@ def download(id: str):
                         type: string
     """
     try:
+        db = Control.instance().db
+        repository = Control.instance().repository
         if repository is not None:
-            model = model_store.get(id, use_typing=False)
-            model_id = model["model"]
+            model = db.model_store.get(id)
+            if model is None:
+                return jsonify({"message": f"Entity with id: {id} not found"}), 404
 
-            file = repository.get_artifact_stream(model_id, modelstorage_config["storage_config"]["storage_bucket"])
+            file = repository.get_model_stream(model.model_id)
 
-            return send_file(file, as_attachment=True, download_name=model_id)
+            return send_file(file, as_attachment=True, download_name=model.model_id)
         else:
             return jsonify({"message": "No model storage configured"}), 500
-    except EntityNotFound:
-        return jsonify({"message": f"Entity with id: {id} not found"}), 404
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -685,11 +747,14 @@ def get_parameters(id: str):
                         type: string
     """
     try:
+        db = Control.instance().db
+        repository = Control.instance().repository
         if repository is not None:
-            model = model_store.get(id, use_typing=False)
-            model_id = model["model"]
+            model = db.model_store.get(id)
+            if model is None:
+                return jsonify({"message": f"Entity with id: {id} not found"}), 404
 
-            file = repository.get_artifact_stream(model_id, modelstorage_config["storage_config"]["storage_bucket"])
+            file = repository.get_model_stream(model.model_id)
 
             file_bytes = io.BytesIO()
             for chunk in file.stream(32 * 1024):
@@ -705,25 +770,30 @@ def get_parameters(id: str):
             return jsonify(array=weights), 200
         else:
             return jsonify({"message": "No model storage configured"}), 500
-    except EntityNotFound:
-        return jsonify({"message": f"Entity with id: {id} not found"}), 404
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
-@bp.route("/active", methods=["GET"])
+@bp.route("/", methods=["POST"])
 @jwt_auth_required(role="admin")
-def get_active_model():
-    """Get active model
-    Retrieves the active model (id).
+def upload_model():
+    """Upload model
+    Uploads a model to the storage backend.
     ---
     tags:
         - Models
+    parameters:
+      - name: model
+        in: body
+        required: true
+        type: object
+        description: The model data to upload
     responses:
         200:
-            description: The active model id
+            description: The uploaded model
             schema:
-                type: string
+                $ref: '#/definitions/Model'
         500:
             description: An error occurred
             schema:
@@ -733,12 +803,64 @@ def get_active_model():
                         type: string
     """
     try:
-        active_model = model_store.get_active()
+        control = Control.instance()
+        data = request.form.to_dict()
+        file = request.files["file"]
+        name: str = data.get("name", None)
 
-        response = active_model
+        try:
+            object = BytesIO()
+            object.seek(0, 0)
+            file.seek(0)
+            object.write(file.read())
+            helper = control.get_helper()
+            logger.info(f"Loading model from file using helper {helper.name}")
+            object.seek(0)
+            model = helper.load(object)
+            control.commit(model=model, name=name)
+        except Exception as e:
+            logger.error(f"An unexpected error occurred: {e}")
+            status_code = 400
+            return (
+                jsonify(
+                    {
+                        "success": False,
+                        "message": "Failed to add model.",
+                    }
+                ),
+                status_code,
+            )
 
-        return jsonify(response), 200
-    except EntityNotFound:
-        return jsonify({"message": "No active model found"}), 404
-    except Exception:
+        return jsonify(
+            {
+                "success": True,
+                "message": "Model added successfully",
+            }
+        ), 200
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
+
+
+# deprecated
+
+
+@bp.route("/active", methods=["GET"])
+@jwt_auth_required(role="admin")
+def get_active_model():
+    return jsonify(
+        {
+            "error": "This endpoint has been deprecated and is no longer available."
+            + "The active model concept is no longer used in Fedn. Please use the /models endpoint to retrieve models.",
+        }
+    ), 410
+
+
+@bp.route("/active", methods=["PUT"])
+@jwt_auth_required(role="admin")
+def set_active_model():
+    return jsonify(
+        {
+            "error": "This endpoint has been deprecated and is no longer available. " + "The active model concept is no longer used in Fedn.",
+        }
+    ), 410
