@@ -1,14 +1,12 @@
 from flask import Blueprint, jsonify, request
 
+from fedn.common.log_config import logger
 from fedn.network.api.auth import jwt_auth_required
-from fedn.network.api.v1.shared import api_version, get_post_data_to_kwargs, get_typed_list_headers, mdb
-from fedn.network.storage.statestore.stores.round_store import RoundStore
-from fedn.network.storage.statestore.stores.shared import EntityNotFound
+from fedn.network.api.v1.shared import api_version, get_post_data_to_kwargs, get_typed_list_headers
+from fedn.network.controller.control import Control
 
 bp = Blueprint("round", __name__, url_prefix=f"/api/{api_version}/rounds")
 
-round_store = RoundStore(mdb, "control.rounds")
-
 
 @bp.route("/", methods=["GET"])
 @jwt_auth_required(role="admin")
@@ -90,18 +88,18 @@ def get_rounds():
                     type: string
     """
     try:
-        limit, skip, sort_key, sort_order, _ = get_typed_list_headers(request.headers)
+        db = Control.instance().db
+        limit, skip, sort_key, sort_order = get_typed_list_headers(request.headers)
 
         kwargs = request.args.to_dict()
 
-        rounds = round_store.list(limit, skip, sort_key, sort_order, use_typing=False, **kwargs)
-
-        result = rounds["result"]
-
-        response = {"count": rounds["count"], "result": result}
+        rounds = db.round_store.list(limit, skip, sort_key, sort_order, **kwargs)
+        count = db.round_store.count(**kwargs)
+        response = {"count": count, "result": [rounds.to_dict() for rounds in rounds]}
 
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -169,18 +167,18 @@ def list_rounds():
                     type: string
     """
     try:
-        limit, skip, sort_key, sort_order, _ = get_typed_list_headers(request.headers)
+        db = Control.instance().db
+        limit, skip, sort_key, sort_order = get_typed_list_headers(request.headers)
 
         kwargs = get_post_data_to_kwargs(request)
 
-        rounds = round_store.list(limit, skip, sort_key, sort_order, use_typing=False, **kwargs)
-
-        result = rounds["result"]
-
-        response = {"count": rounds["count"], "result": result}
+        rounds = db.round_store.list(limit, skip, sort_key, sort_order, **kwargs)
+        count = db.round_store.count(**kwargs)
+        response = {"count": count, "result": [round.to_dict() for round in rounds]}
 
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -217,11 +215,13 @@ def get_rounds_count():
                         type: string
     """
     try:
+        db = Control.instance().db
         kwargs = request.args.to_dict()
-        count = round_store.count(**kwargs)
+        count = db.round_store.count(**kwargs)
         response = count
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -262,11 +262,13 @@ def rounds_count():
                         type: string
     """
     try:
+        db = Control.instance().db
         kwargs = get_post_data_to_kwargs(request)
-        count = round_store.count(**kwargs)
+        count = db.round_store.count(**kwargs)
         response = count
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -305,11 +307,12 @@ def get_round(id: str):
                         type: string
     """
     try:
-        round = round_store.get(id, use_typing=False)
-        response = round
-
+        db = Control.instance().db
+        round = db.round_store.get(id)
+        if round is None:
+            return jsonify({"message": f"Entity with id: {id} not found"}), 404
+        response = round.to_dict()
         return jsonify(response), 200
-    except EntityNotFound:
-        return jsonify({"message": f"Entity with id: {id} not found"}), 404
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
