@@ -2,20 +2,17 @@ import threading
 
 from flask import Blueprint, jsonify, request
 
+from fedn.common.log_config import logger
 from fedn.network.api.auth import jwt_auth_required
-from fedn.network.api.shared import control
-from fedn.network.api.v1.shared import api_version, get_post_data_to_kwargs, get_typed_list_headers, mdb
+from fedn.network.api.v1.shared import api_version, get_post_data_to_kwargs, get_typed_list_headers
 from fedn.network.combiner.interfaces import CombinerUnavailableError
+from fedn.network.controller.control import Control
 from fedn.network.state import ReducerState
-from fedn.network.storage.statestore.stores.session_store import SessionStore
-from fedn.network.storage.statestore.stores.shared import EntityNotFound
-
-from .model_routes import model_store
+from fedn.network.storage.statestore.stores.dto.session import SessionConfigDTO, SessionDTO
+from fedn.network.storage.statestore.stores.shared import EntityNotFound, MissingFieldError, ValidationError
 
 bp = Blueprint("session", __name__, url_prefix=f"/api/{api_version}/sessions")
 
-session_store = SessionStore(mdb, "control.sessions")
-
 
 @bp.route("/", methods=["GET"])
 @jwt_auth_required(role="admin")
@@ -90,17 +87,19 @@ def get_sessions():
                     type: string
     """
     try:
-        limit, skip, sort_key, sort_order, _ = get_typed_list_headers(request.headers)
+        db = Control.instance().db
+        limit, skip, sort_key, sort_order = get_typed_list_headers(request.headers)
         kwargs = request.args.to_dict()
 
-        sessions = session_store.list(limit, skip, sort_key, sort_order, use_typing=False, **kwargs)
+        sessions = db.session_store.list(limit, skip, sort_key, sort_order, **kwargs)
 
-        result = sessions["result"]
-
-        response = {"count": sessions["count"], "result": result}
+        count = db.session_store.count(**kwargs)
+        result = [session.to_dict() for session in sessions]
 
+        response = {"count": count, "result": result}
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -168,17 +167,20 @@ def list_sessions():
                     type: string
     """
     try:
-        limit, skip, sort_key, sort_order, _ = get_typed_list_headers(request.headers)
+        db = Control.instance().db
+        limit, skip, sort_key, sort_order = get_typed_list_headers(request.headers)
         kwargs = get_post_data_to_kwargs(request)
 
-        sessions = session_store.list(limit, skip, sort_key, sort_order, use_typing=False, **kwargs)
+        sessions = db.session_store.list(limit, skip, sort_key, sort_order, **kwargs)
 
-        result = sessions["result"]
+        count = db.session_store.count(**kwargs)
+        result = [session.to_dict() for session in sessions]
 
-        response = {"count": sessions["count"], "result": result}
+        response = {"count": count, "result": result}
 
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -215,11 +217,13 @@ def get_sessions_count():
                         type: string
     """
     try:
+        db = Control.instance().db
         kwargs = request.args.to_dict()
-        count = session_store.count(**kwargs)
+        count = db.session_store.count(**kwargs)
         response = count
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -260,11 +264,13 @@ def sessions_count():
                         type: string
     """
     try:
+        db = Control.instance().db
         kwargs = get_post_data_to_kwargs(request)
-        count = session_store.count(**kwargs)
+        count = db.session_store.count(**kwargs)
         response = count
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -303,13 +309,14 @@ def get_session(id: str):
                         type: string
     """
     try:
-        session = session_store.get(id, use_typing=False)
-        response = session
-
+        db = Control.instance().db
+        result = db.session_store.get(id)
+        if result is None:
+            return jsonify({"message": f"Entity with id: {id} not found"}), 404
+        response = result.to_dict()
         return jsonify(response), 200
-    except EntityNotFound:
-        return jsonify({"message": f"Entity with id: {id} not found"}), 404
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -346,22 +353,52 @@ def post():
                         type: string
     """
     try:
-        data = request.json if request.headers["Content-Type"] == "application/json" else request.form.to_dict()
-        successful, result = session_store.add(data)
-        response = result
-        status_code: int = 201 if successful else 400
+        db = Control.instance().db
+        data = request.get_json(silent=True) if request.is_json else request.form.to_dict()
+
+        session_config = SessionConfigDTO()
+        session_config.populate_with(data.pop("session_config"))
+
+        session = SessionDTO()
+        session.session_id = None
+        session.session_config = session_config
+        session.populate_with(data)
+
+        result = db.session_store.add(session)
+
+        response = result.to_dict()
+        status_code: int = 201
 
         return jsonify(response), status_code
-    except Exception:
+    except ValidationError as e:
+        logger.error(f"Validation error: {e}")
+        return jsonify({"message": e.user_message()}), 400
+    except MissingFieldError as e:
+        logger.error(f"Missing field error: {e}")
+        return jsonify({"message": e.user_message()}), 400
+    except ValueError as e:
+        logger.error(f"ValueError occurred: {e}")
+        return jsonify({"message": "Invalid object"}), 400
+    except Exception as e:
+        logger.error("error when creating a session")
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
-def _get_number_of_available_clients():
+def _get_number_of_available_clients(client_ids: list[str]):
+    control = Control.instance()
+
     result = 0
+    active_clients = None
     for combiner in control.network.get_combiners():
         try:
-            nr_active_clients = len(combiner.list_active_clients())
-            result = result + int(nr_active_clients)
+            active_clients = combiner.list_active_clients()
+            if active_clients is not None:
+                if client_ids is not None:
+                    filtered = [item for item in active_clients if item.client_id in client_ids]
+                    result += len(filtered)
+                else:
+                    result += len(active_clients)
         except CombinerUnavailableError:
             return 0
 
@@ -378,37 +415,103 @@ def start_session():
     type: rounds: int
     """
     try:
-        data = request.json if request.headers["Content-Type"] == "application/json" else request.form.to_dict()
+        db = Control.instance().db
+        control = Control.instance()
+
+        data = request.get_json(silent=True) if request.is_json else request.form.to_dict()
+
         session_id: str = data.get("session_id")
         rounds: int = data.get("rounds", "")
         round_timeout: int = data.get("round_timeout", None)
+        model_name_prefix: str = data.get("model_name_prefix", None)
+        client_ids: str = data.get("client_ids", None)
+
+        if client_ids is not None and not isinstance(client_ids, str):
+            return jsonify({"message": "client_ids must be a comma separated string"}), 400
+        if client_ids is not None:
+            client_ids: list[str] = client_ids.split(",")
+            if len(client_ids) == 0:
+                return jsonify({"message": "client_ids must be a comma separated string"}), 400
+            if any(not isinstance(client_id, str) for client_id in client_ids):
+                return jsonify({"message": "client_ids must be a comma separated string"}), 400
+
+        if model_name_prefix is None or not isinstance(model_name_prefix, str) or len(model_name_prefix) == 0:
+            model_name_prefix = None
 
         if not session_id or session_id == "":
             return jsonify({"message": "Session ID is required"}), 400
 
-        session = session_store.get(session_id, use_typing=False)
+        session = db.session_store.get(session_id)
 
-        session_config = session["session_config"]
-        model_id = session_config["model_id"]
-        min_clients = session_config["clients_required"]
+        if not session:
+            return jsonify({"message": "Session with specified ID does not exist"}), 400
+
+        session_config = session.session_config
+        model_id = session_config.model_id
+        min_clients = session_config.clients_required
 
         if control.state() == ReducerState.monitoring:
-            return jsonify({"message": "A session is already running."})
+            return jsonify({"message": "A session is already running!"}), 400
 
         if not rounds or not isinstance(rounds, int):
-            rounds = session_config["rounds"]
+            rounds = session_config.rounds
+        nr_available_clients = _get_number_of_available_clients(client_ids=client_ids)
+
+        if nr_available_clients < min_clients:
+            return jsonify({"message": f"Number of available clients is lower than the required minimum of {min_clients}"}), 400
+
+        model = db.model_store.get(model_id)
+        if model is None:
+            return jsonify({"message": "Session seed model not found"}), 400
+
+        threading.Thread(target=control.start_session, args=(session_id, rounds, round_timeout, model_name_prefix, client_ids)).start()
+
+        return jsonify({"message": "Session started"}), 200
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
+        return jsonify({"message": "An unexpected error occurred"}), 500
+
+
+@bp.route("/start_splitlearning_session", methods=["POST"])
+@jwt_auth_required(role="admin")
+def start_splitlearning_session():
+    """Starts a new split learning session."""
+    try:
+        db = Control.instance().db
+        control = Control.instance()
+        data = request.json if request.headers["Content-Type"] == "application/json" else request.form.to_dict()
+        session_id: str = data.get("session_id")
+        rounds: int = data.get("rounds", "")
+        round_timeout: int = data.get("round_timeout", None)
+        model_name_prefix: str = data.get("model_name_prefix", None)
+
+        if model_name_prefix is None or not isinstance(model_name_prefix, str) or len(model_name_prefix) == 0:
+            model_name_prefix = None
+
+        if not session_id or session_id == "":
+            return jsonify({"message": "Session ID is required"}), 400
+
+        session = db.session_store.get(session_id)
+        session_config = session.session_config
+        min_clients = session_config.clients_required
+
+        if control.state() == ReducerState.monitoring:
+            return jsonify({"message": "A session is already running!"}), 400
+
+        if not rounds or not isinstance(rounds, int):
+            rounds = session_config.rounds
         nr_available_clients = _get_number_of_available_clients()
 
         if nr_available_clients < min_clients:
             return jsonify({"message": f"Number of available clients is lower than the required minimum of {min_clients}"}), 400
 
-        _ = model_store.get(model_id, use_typing=False)
+        threading.Thread(target=control.splitlearning_session, args=(session_id, rounds, round_timeout)).start()
 
-        threading.Thread(target=control.start_session, args=(session_id, rounds, round_timeout)).start()
+        return jsonify({"message": "Splitlearning session started"}), 200
+    except Exception as e:
+        logger.error(f"An unexpected error occurred in split learning session: {e}")
+        return jsonify({"message": "An unexpected error occurred when starting split learning session"}), 500
 
-        return jsonify({"message": "Session started"}), 200
-    except Exception:
-        return jsonify({"message": "An unexpected error occurred"}), 500
 
 @bp.route("/<string:id>", methods=["PATCH"])
 @jwt_auth_required(role="admin")
@@ -450,28 +553,36 @@ def patch_session(id: str):
                         type: string
     """
     try:
-        session = session_store.get(id, use_typing=False)
+        db = Control.instance().db
+        existing_session = db.session_store.get(id)
+        if existing_session is None:
+            return jsonify({"message": f"Entity with id: {id} not found"}), 404
 
         data = request.get_json()
-        _id = session["id"]
+        # Remove session_id from the data if it exists
+        # since we are editing 'id' otherwise the user could change the id
+        data.pop("session_id", None)
 
-        # Update the session with the new data
-        # Only update the fields that are present in the request
-        for key, value in data.items():
-            if key in ["_id", "session_id"]:
-                continue
-            session[key] = value
+        existing_session.patch_with(data, throw_on_extra_keys=False)
+        updated_session = db.session_store.update(existing_session)
 
-        success, message = session_store.update(_id, session)
+        response = updated_session.to_dict()
+        return jsonify(response), 200
 
-        if success:
-            response = session
-            return jsonify(response), 200
-
-        return jsonify({"message": f"Failed to update session: {message}"}), 500
-    except EntityNotFound:
+    except EntityNotFound as e:
+        logger.error(f"Entity not found: {e}")
         return jsonify({"message": f"Entity with id: {id} not found"}), 404
-    except Exception:
+    except ValidationError as e:
+        logger.error(f"Validation error: {e}")
+        return jsonify({"message": e.user_message()}), 400
+    except MissingFieldError as e:
+        logger.error(f"Missing field error: {e}")
+        return jsonify({"message": e.user_message()}), 400
+    except ValueError as e:
+        logger.error(f"ValueError occured: {e}")
+        return jsonify({"message": "Invalid object"}), 400
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -515,18 +626,32 @@ def put_session(id: str):
                         type: string
     """
     try:
-        session = session_store.get(id, use_typing=False)
+        db = Control.instance().db
+        session = db.session_store.get(id)
+        if session is None:
+            return jsonify({"message": f"Entity with id: {id} not found"}), 404
+
         data = request.get_json()
-        _id = session["id"]
+        data["session_id"] = id
+        session.populate_with(data)
 
-        success, message = session_store.update(_id, data)
+        updated_session = db.session_store.update(session)
 
-        if success:
-            response = session
-            return jsonify(response), 200
+        response = updated_session.to_dict()
+        return jsonify(response), 200
 
-        return jsonify({"message": f"Failed to update session: {message}"}), 500
-    except EntityNotFound:
+    except EntityNotFound as e:
+        logger.error(f"Entity not found: {e}")
         return jsonify({"message": f"Entity with id: {id} not found"}), 404
-    except Exception:
+    except ValidationError as e:
+        logger.error(f"Validation error: {e}")
+        return jsonify({"message": e.user_message()}), 400
+    except MissingFieldError as e:
+        logger.error(f"Missing field error: {e}")
+        return jsonify({"message": e.user_message()}), 400
+    except ValueError as e:
+        logger.error(f"ValueError occured: {e}")
+        return jsonify({"message": "Invalid object"}), 400
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
