@@ -1,25 +1,14 @@
 from typing import Tuple
 
-import pymongo
-from pymongo.database import Database
-
-from fedn.network.api.shared import statestore_config,network_id
+from fedn.network.storage.statestore.stores.shared import SortOrder
 
 api_version = "v1"
-mc = pymongo.MongoClient(**statestore_config["mongo_config"])
-mc.server_info()
-mdb: Database = mc[network_id]
 
 
 def is_positive_integer(s):
     return s is not None and s.isdigit() and int(s) > 0
 
 
-def get_use_typing(headers: object) -> bool:
-    skip_typing: str = headers.get("X-Skip-Typing", "false")
-    return False if skip_typing.lower() == "true" else True
-
-
 def get_limit(headers: object) -> int:
     limit: str = headers.get("X-Limit")
     if is_positive_integer(limit):
@@ -49,25 +38,36 @@ def get_typed_list_headers(
 
     limit: int = get_limit(headers)
     skip: int = get_skip(headers)
-    use_typing: bool = get_use_typing(headers)
 
     if sort_order is not None:
-        sort_order = pymongo.ASCENDING if sort_order.lower() == "asc" else pymongo.DESCENDING
+        sort_order = SortOrder.ASCENDING if sort_order.lower() == "asc" else SortOrder.DESCENDING
     else:
-        sort_order = pymongo.DESCENDING
+        sort_order = SortOrder.DESCENDING
 
-    return limit, skip, sort_key, sort_order, use_typing
+    return limit, skip, sort_key, sort_order
 
 
 def get_post_data_to_kwargs(request: object) -> dict:
-    request_data = request.form.to_dict()
+    try:
+        # Try to get data from form
+        request_data = request.form.to_dict()
+    except Exception:
+        request_data = None
 
     if not request_data:
-        request_data = request.json
+        try:
+            # Try to get data from JSON
+            request_data = request.get_json()
+        except Exception:
+            request_data = {}
+
+    # Ensure request_data is a dictionary
+    if not isinstance(request_data, dict):
+        request_data = {}
 
     kwargs = {}
     for key, value in request_data.items():
-        if "," in value:
+        if isinstance(value, str) and "," in value:
             kwargs[key] = {"$in": value.split(",")}
         else:
             kwargs[key] = value
