@@ -1,14 +1,12 @@
 from flask import Blueprint, jsonify, request
 
+from fedn.common.log_config import logger
 from fedn.network.api.auth import jwt_auth_required
-from fedn.network.api.v1.shared import api_version, get_post_data_to_kwargs, get_typed_list_headers, get_use_typing, mdb
-from fedn.network.storage.statestore.stores.shared import EntityNotFound
-from fedn.network.storage.statestore.stores.status_store import StatusStore
+from fedn.network.api.v1.shared import api_version, get_post_data_to_kwargs, get_typed_list_headers
+from fedn.network.controller.control import Control
 
 bp = Blueprint("status", __name__, url_prefix=f"/api/{api_version}/statuses")
 
-status_store = StatusStore(mdb, "control.status")
-
 
 @bp.route("/", methods=["GET"])
 @jwt_auth_required(role="admin")
@@ -121,17 +119,17 @@ def get_statuses():
                     type: string
     """
     try:
-        limit, skip, sort_key, sort_order, use_typing = get_typed_list_headers(request.headers)
+        db = Control.instance().db
+        limit, skip, sort_key, sort_order = get_typed_list_headers(request.headers)
         kwargs = request.args.to_dict()
 
-        statuses = status_store.list(limit, skip, sort_key, sort_order, use_typing=use_typing, **kwargs)
-
-        result = [status.__dict__ for status in statuses["result"]] if use_typing else statuses["result"]
-
-        response = {"count": statuses["count"], "result": result}
+        result = db.status_store.list(limit, skip, sort_key, sort_order, **kwargs)
+        count = db.status_store.count(**kwargs)
+        response = {"count": count, "result": [item.to_dict() for item in result]}
 
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -216,17 +214,17 @@ def list_statuses():
                     type: string
     """
     try:
-        limit, skip, sort_key, sort_order, use_typing = get_typed_list_headers(request.headers)
+        db = Control.instance().db
+        limit, skip, sort_key, sort_order = get_typed_list_headers(request.headers)
         kwargs = get_post_data_to_kwargs(request)
 
-        statuses = status_store.list(limit, skip, sort_key, sort_order, use_typing=use_typing, **kwargs)
-
-        result = [status.__dict__ for status in statuses["result"]] if use_typing else statuses["result"]
-
-        response = {"count": statuses["count"], "result": result}
+        result = db.status_store.list(limit, skip, sort_key, sort_order, **kwargs)
+        count = db.status_store.count(**kwargs)
+        response = {"count": count, "result": [item.to_dict() for item in result]}
 
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -284,11 +282,13 @@ def get_statuses_count():
                         type: string
     """
     try:
+        db = Control.instance().db
         kwargs = request.args.to_dict()
-        count = status_store.count(**kwargs)
+        count = db.status_store.count(**kwargs)
         response = count
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -346,11 +346,13 @@ def statuses_count():
                         type: string
     """
     try:
+        db = Control.instance().db
         kwargs = get_post_data_to_kwargs(request)
-        count = status_store.count(**kwargs)
+        count = db.status_store.count(**kwargs)
         response = count
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -389,13 +391,11 @@ def get_status(id: str):
                         type: string
     """
     try:
-        use_typing: bool = get_use_typing(request.headers)
-        status = status_store.get(id, use_typing=use_typing)
-
-        response = status.__dict__ if use_typing else status
-
-        return jsonify(response), 200
-    except EntityNotFound:
-        return jsonify({"message": f"Entity with id: {id} not found"}), 404
-    except Exception:
+        db = Control.instance().db
+        status = db.status_store.get(id)
+        if status is None:
+            return jsonify({"message": f"Entity with id: {id} not found"}), 404
+        return jsonify(status.to_dict()), 200
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
