@@ -1,14 +1,12 @@
 from flask import Blueprint, jsonify, request
 
+from fedn.common.log_config import logger
 from fedn.network.api.auth import jwt_auth_required
-from fedn.network.api.v1.shared import api_version, get_post_data_to_kwargs, get_typed_list_headers, get_use_typing, mdb
-from fedn.network.storage.statestore.stores.shared import EntityNotFound
-from fedn.network.storage.statestore.stores.validation_store import ValidationStore
+from fedn.network.api.v1.shared import api_version, get_post_data_to_kwargs, get_typed_list_headers
+from fedn.network.controller.control import Control
 
 bp = Blueprint("validation", __name__, url_prefix=f"/api/{api_version}/validations")
 
-validation_store = ValidationStore(mdb, "control.validations")
-
 
 @bp.route("/", methods=["GET"])
 @jwt_auth_required(role="admin")
@@ -128,17 +126,17 @@ def get_validations():
                     type: string
     """
     try:
-        limit, skip, sort_key, sort_order, use_typing = get_typed_list_headers(request.headers)
+        db = Control.instance().db
+        limit, skip, sort_key, sort_order = get_typed_list_headers(request.headers)
         kwargs = request.args.to_dict()
 
-        validations = validation_store.list(limit, skip, sort_key, sort_order, use_typing=use_typing, **kwargs)
-
-        result = [validation.__dict__ for validation in validations["result"]] if use_typing else validations["result"]
-
-        response = {"count": validations["count"], "result": result}
+        result = db.validation_store.list(limit, skip, sort_key, sort_order, **kwargs)
+        count = db.validation_store.count(**kwargs)
+        response = {"count": count, "result": [validation.to_dict() for validation in result]}
 
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -226,17 +224,17 @@ def list_validations():
                     type: string
     """
     try:
-        limit, skip, sort_key, sort_order, use_typing = get_typed_list_headers(request.headers)
+        db = Control.instance().db
+        limit, skip, sort_key, sort_order = get_typed_list_headers(request.headers)
         kwargs = get_post_data_to_kwargs(request)
 
-        validations = validation_store.list(limit, skip, sort_key, sort_order, use_typing=use_typing, **kwargs)
-
-        result = [validation.__dict__ for validation in validations["result"]] if use_typing else validations["result"]
-
-        response = {"count": validations["count"], "result": result}
+        result = db.validation_store.list(limit, skip, sort_key, sort_order, **kwargs)
+        count = db.validation_store.count(**kwargs)
+        response = {"count": count, "result": [validation.to_dict() for validation in result]}
 
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -298,11 +296,13 @@ def get_validations_count():
                         type: string
     """
     try:
+        db = Control.instance().db
         kwargs = request.args.to_dict()
-        count = validation_store.count(**kwargs)
+        count = db.validation_store.count(**kwargs)
         response = count
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -363,11 +363,13 @@ def validations_count():
                         type: string
     """
     try:
+        db = Control.instance().db
         kwargs = get_post_data_to_kwargs(request)
-        count = validation_store.count(**kwargs)
+        count = db.validation_store.count(**kwargs)
         response = count
         return jsonify(response), 200
-    except Exception:
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
 
 
@@ -406,13 +408,11 @@ def get_validation(id: str):
                         type: string
     """
     try:
-        use_typing: bool = get_use_typing(request.headers)
-        validation = validation_store.get(id, use_typing=use_typing)
-
-        response = validation.__dict__ if use_typing else validation
-
-        return jsonify(response), 200
-    except EntityNotFound:
-        return jsonify({"message": f"Entity with id: {id} not found"}), 404
-    except Exception:
+        db = Control.instance().db
+        response = db.validation_store.get(id)
+        if response is None:
+            return jsonify({"message": f"Entity with id: {id} not found"}), 404
+        return jsonify(response.to_dict()), 200
+    except Exception as e:
+        logger.error(f"An unexpected error occurred: {e}")
         return jsonify({"message": "An unexpected error occurred"}), 500
