@@ -1,3 +1,5 @@
+"""Module for representing and converting client states."""
+
 from enum import Enum
 
 
@@ -7,10 +9,10 @@ class ClientState(Enum):
     idle = 1
     training = 2
     validating = 3
-    inferencing = 4
+    predicting = 4
 
 
-def ClientStateToString(state):
+def client_state_to_string(state: ClientState) -> str:
     """Convert a ClientState to a string representation.
 
     :param state: the state to convert
@@ -24,5 +26,7 @@ def ClientStateToString(state):
         return "TRAINING"
     if state == ClientState.validating:
         return "VALIDATING"
+    if state == ClientState.predicting:
+        return "PREDICTING"
 
     return "UNKNOWN"
