@@ -18,7 +18,7 @@ class TestFedAvg(unittest.TestCase):
     def test_fedavg_combine_models(self, *args, **kwargs):
         """Test the FedAvg aggregator combine_models method with mock classes and methods"""
         aggregator = FedAvg("id", None, None, None, None)
-        aggregator.next_model_update = MagicMock(return_value=(None, None, None))
+        aggregator.update_handler.next_model_update = MagicMock(return_value=[(None, None, None)])
         aggregator.server = MagicMock()
 
         data = {}
