@@ -1,24 +1,34 @@
-import base64
 import json
 import queue
 import re
 import signal
-import sys
 import threading
 import time
 import uuid
 from datetime import datetime, timedelta
 from enum import Enum
+from typing import TypedDict
+
+from google.protobuf.json_format import MessageToDict
 
 import fedn.network.grpc.fedn_pb2 as fedn
 import fedn.network.grpc.fedn_pb2_grpc as rpc
+from fedn.common.certificate.certificate import Certificate
 from fedn.common.log_config import logger, set_log_level_from_string, set_log_stream
-from fedn.network.combiner.connect import ConnectorCombiner, Status
 from fedn.network.combiner.modelservice import ModelService
 from fedn.network.combiner.roundhandler import RoundConfig, RoundHandler
-from fedn.network.grpc.server import Server
+from fedn.network.grpc.server import Server, ServerConfig
+from fedn.network.storage.dbconnection import DatabaseConnection
 from fedn.network.storage.s3.repository import Repository
-from fedn.network.storage.statestore.mongostatestore import MongoStateStore
+from fedn.network.storage.statestore.stores.dto import ClientDTO
+from fedn.network.storage.statestore.stores.dto.attribute import AttributeDTO
+from fedn.network.storage.statestore.stores.dto.combiner import CombinerDTO
+from fedn.network.storage.statestore.stores.dto.metric import MetricDTO
+from fedn.network.storage.statestore.stores.dto.prediction import PredictionDTO
+from fedn.network.storage.statestore.stores.dto.status import StatusDTO
+from fedn.network.storage.statestore.stores.dto.telemetry import TelemetryDTO
+from fedn.network.storage.statestore.stores.dto.validation import ValidationDTO
+from fedn.network.storage.statestore.stores.shared import SortOrder
 
 VALID_NAME_REGEX = "^[a-zA-Z0-9_-]*$"
 
@@ -43,13 +53,36 @@ def role_to_proto_role(role):
     if role == Role.COMBINER:
         return fedn.COMBINER
     if role == Role.WORKER:
-        return fedn.WORKER
+        return fedn.CLIENT
     if role == Role.REDUCER:
         return fedn.REDUCER
     if role == Role.OTHER:
         return fedn.OTHER
 
 
+class CombinerConfig(TypedDict):
+    """Configuration for the combiner."""
+
+    discover_host: str
+    discover_port: int
+    token: str
+    host: str
+    port: int
+    ip: str
+    parent: str
+    fqdn: str
+    name: str
+    secure: bool
+    verify: bool
+    cert_path: str
+    key_path: str
+    max_clients: int
+    network_id: str
+    logfile: str
+    verbosity: str
+
+
+# TODO: dependency injection
 class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer, rpc.ControlServicer):
     """Combiner gRPC server.
 
@@ -57,12 +90,20 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
     :type config: dict
     """
 
-    def __init__(self, config):
+    _instance: "Combiner"
+
+    def __init__(self, config, repository: Repository, db: DatabaseConnection):
         """Initialize Combiner server."""
         set_log_level_from_string(config.get("verbosity", "INFO"))
         set_log_stream(config.get("logfile", None))
 
         # Client queues
+        # Each client in the dict is stored with its client_id as key, and the value is a dict with keys:
+        #     name: str
+        #     status: str
+        #     last_seen: str
+        #     fedn.Queue.TASK_QUEUE: queue.Queue
+        # Obs that fedn.Queue.TASK_QUEUE is just str(1)
         self.clients = {}
 
         # Validate combiner name
@@ -74,68 +115,52 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         self.role = Role.COMBINER
         self.max_clients = config["max_clients"]
 
-        # Connector to announce combiner to discover service (reducer)
-        announce_client = ConnectorCombiner(
-            host=config["discover_host"],
-            port=config["discover_port"],
-            myhost=config["host"],
-            fqdn=config["fqdn"],
-            myport=config["port"],
-            token=config["token"],
-            name=config["name"],
-            secure=config["secure"],
-            verify=config["verify"],
-        )
-
-        while True:
-            # Announce combiner to discover service
-            status, response = announce_client.announce()
-            if status == Status.TryAgain:
-                logger.info(response)
-                time.sleep(5)
-            elif status == Status.Assigned:
-                announce_config = response
-                logger.info("COMBINER {0}: Announced successfully".format(self.id))
-                break
-            elif status == Status.UnAuthorized:
-                logger.info(response)
-                logger.info("Status.UnAuthorized")
-                sys.exit("Exiting: Unauthorized")
-            elif status == Status.UnMatchedConfig:
-                logger.info(response)
-                logger.info("Status.UnMatchedConfig")
-                sys.exit("Exiting: Missing config")
-
-        cert = announce_config["certificate"]
-        key = announce_config["key"]
-
-        if announce_config["certificate"]:
-            cert = base64.b64decode(announce_config["certificate"])  # .decode('utf-8')
-            key = base64.b64decode(announce_config["key"])  # .decode('utf-8')
-
-        # Set up gRPC server configuration
-        grpc_config = {"port": config["port"], "secure": config["secure"], "certificate": cert, "key": key}
-
         # Set up model repository
-        self.repository = Repository(announce_config["storage"]["storage_config"])
+        self.repository = repository
+        self.db = db
 
-        self.statestore = MongoStateStore(announce_config["statestore"]["network_id"], announce_config["statestore"]["mongo_config"])
+        # Check if combiner already exists in statestore
+        if self.db.combiner_store.get_by_name(config["name"]) is None:
+            new_combiner = CombinerDTO()
+            new_combiner.port = config["port"]
+            new_combiner.fqdn = config["fqdn"]
+            new_combiner.name = config["name"]
+            new_combiner.address = config["host"]
+            new_combiner.parent = "localhost"
+            new_combiner.ip = ""
+            new_combiner.updated_at = str(datetime.now())
+            self.db.combiner_store.add(new_combiner)
 
         # Fetch all clients previously connected to the combiner
         # If a client and a combiner goes down at the same time,
         # the client will be stuck listed as "online" in the statestore.
         # Set the status to offline for previous clients.
-        previous_clients = self.statestore.clients.find({"combiner": config["name"]})
+        previous_clients = self.db.client_store.list(limit=0, skip=0, sort_key=None, sort_order=SortOrder.DESCENDING, combiner=self.id)
+        logger.info(f"Found {len(previous_clients)} previous clients")
+        logger.info("Updating previous clients status to offline")
         for client in previous_clients:
-            self.statestore.set_client({"name": client["name"], "status": "offline", "client_id": client["client_id"]})
+            client.status = "offline"
+            try:
+                self.db.client_store.update(client)
+            except Exception as e:
+                logger.error(f"Failed to update previous client status: {e}")
 
-        self.modelservice = ModelService()
+        # Set up gRPC server configuration
+        if config["secure"]:
+            cert = Certificate(key_path=config["key_path"], cert_path=config["cert_path"])
+            certificate, key = cert.get_keypair_raw()
+            grpc_server_config = ServerConfig(port=config["port"], secure=True, key=key, certificate=certificate)
+        else:
+            grpc_server_config = ServerConfig(port=config["port"], secure=False)
+
+        # Set up model service
+        modelservice = ModelService()
 
         # Create gRPC server
-        self.server = Server(self, self.modelservice, grpc_config)
+        self.server = Server(self, modelservice, grpc_server_config)
 
         # Set up round controller
-        self.round_handler = RoundHandler(self.repository, self, self.modelservice)
+        self.round_handler = RoundHandler(self, repository, modelservice)
 
         # Start thread for round controller
         threading.Thread(target=self.round_handler.run, daemon=True).start()
@@ -146,6 +171,25 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         # Start the gRPC server
         self.server.start()
 
+    @classmethod
+    def create_instance(cls, config: CombinerConfig, repository: Repository, db: DatabaseConnection):
+        """Create a new singleton instance of the combiner.
+
+        :param config: configuration for the combiner
+        :type config: dict
+        :return: the instance of the combiner
+        :rtype: :class:`fedn.network.combiner.server.Combiner`
+        """
+        cls._instance = cls(config, repository, db)
+        return cls._instance
+
+    @classmethod
+    def instance(cls):
+        """Get the singleton instance of the combiner."""
+        if cls._instance is None:
+            raise Exception("Combiner instance not created yet.")
+        return cls._instance
+
     def __whoami(self, client, instance):
         """Set the client id and role in a proto message.
 
@@ -194,10 +238,10 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         else:
             logger.info("Sent model validation request for model {} to {} clients".format(model_id, len(clients)))
 
-    def request_model_inference(self, session_id: str, model_id: str, clients: list = []) -> None:
-        """Ask clients to perform inference on the model.
+    def request_model_prediction(self, prediction_id: str, model_id: str, clients: list = []) -> None:
+        """Ask clients to perform prediction on the model.
 
-        :param model_id: the model id to perform inference on
+        :param model_id: the model id to perform prediction on
         :type model_id: str
         :param config: the model configuration to send to clients
         :type config: dict
@@ -205,18 +249,51 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         :type clients: list
 
         """
-        clients = self._send_request_type(fedn.StatusType.INFERENCE, session_id, model_id, clients)
+        clients = self._send_request_type(fedn.StatusType.MODEL_PREDICTION, prediction_id, model_id, {}, clients)
 
         if len(clients) < 20:
-            logger.info("Sent model inference request for model {} to clients {}".format(model_id, clients))
+            logger.info("Sent model prediction request for model {} to clients {}".format(model_id, clients))
         else:
-            logger.info("Sent model inference request for model {} to {} clients".format(model_id, len(clients)))
+            logger.info("Sent model prediction request for model {} to {} clients".format(model_id, len(clients)))
 
-    def _send_request_type(self, request_type, session_id, model_id, config=None, clients=[]):
+    def request_forward_pass(self, session_id: str, model_id: str, config: dict, clients=[]) -> None:
+        """Ask clients to perform forward pass.
+
+        :param config: the model configuration to send to clients
+        :type config: dict
+        :param clients: the clients to send the request to
+        :type clients: list
+
+        """
+        clients = self._send_request_type(fedn.StatusType.FORWARD, session_id, model_id, config, clients)
+
+        if len(clients) < 20:
+            logger.info("Sent forward request to clients {}".format(clients))
+        else:
+            logger.info("Sent forward request to {} clients".format(len(clients)))
+
+    def request_backward_pass(self, session_id: str, gradient_id: str, config: dict, clients=[]) -> None:
+        """Ask clients to perform backward pass.
+
+        :param config: the model configuration to send to clients
+        :type config: dict
+        :param clients: the clients to send the request to
+        :type clients: list
+        """
+        clients = self._send_request_type(fedn.StatusType.BACKWARD, session_id, gradient_id, config, clients)
+
+        if len(clients) < 20:
+            logger.info("Sent backward request for gradients {} to clients {}".format(gradient_id, clients))
+        else:
+            logger.info("Sent backward request for gradients {} to {} clients".format(gradient_id, len(clients)))
+
+    def _send_request_type(self, request_type, session_id, model_id=None, config=None, clients=[]):
         """Send a request of a specific type to clients.
 
         :param request_type: the type of request
         :type request_type: :class:`fedn.network.grpc.fedn_pb2.StatusType`
+        :param session_id: the session id to send in the request. Obs that for prediction, this is the prediction id.
+        :type session_id: str
         :param model_id: the model id to send in the request
         :type model_id: str
         :param config: the model configuration to send to clients
@@ -227,13 +304,14 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         :rtype: list
         """
         if len(clients) == 0:
-            if request_type == fedn.StatusType.MODEL_UPDATE:
+            if request_type in [fedn.StatusType.MODEL_UPDATE, fedn.StatusType.FORWARD, fedn.StatusType.BACKWARD]:
                 clients = self.get_active_trainers()
             elif request_type == fedn.StatusType.MODEL_VALIDATION:
                 clients = self.get_active_validators()
-            elif request_type == fedn.StatusType.INFERENCE:
-                # TODO: add inference clients type
+            elif request_type == fedn.StatusType.MODEL_PREDICTION:
+                # TODO: add prediction clients type
                 clients = self.get_active_validators()
+
         for client in clients:
             request = fedn.TaskRequest()
             request.model_id = model_id
@@ -245,13 +323,14 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
             request.sender.name = self.id
             request.sender.role = fedn.COMBINER
             request.receiver.client_id = client
-            request.receiver.role = fedn.WORKER
+            request.receiver.role = fedn.CLIENT
+
             # Set the request data, not used in validation
-            if request_type == fedn.StatusType.INFERENCE:
-                presigned_url = self.repository.presigned_put_url(self.repository.inference_bucket, f"{client}/{session_id}")
-                # TODO: in inference, request.data should also contain user-defined data/parameters
+            if request_type == fedn.StatusType.MODEL_PREDICTION:
+                presigned_url = self.repository.presigned_put_url(self.repository.prediction_bucket, f"{client}/{session_id}")
+                # TODO: in prediction, request.data should also contain user-defined data/parameters
                 request.data = json.dumps({"presigned_url": presigned_url})
-            elif request_type == fedn.StatusType.MODEL_UPDATE:
+            elif request_type in [fedn.StatusType.MODEL_UPDATE, fedn.StatusType.FORWARD, fedn.StatusType.BACKWARD]:
                 request.data = json.dumps(config)
             self._put_request_to_client_queue(request, fedn.Queue.TASK_QUEUE)
         return clients
@@ -357,7 +436,7 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
             then = self.clients[client]["last_seen"]
             if (now - then) < timedelta(seconds=10):
                 clients["active_clients"].append(client)
-                # If client has changed status, update statestore
+                # If client has changed status, update client queue
                 if status != "online":
                     self.clients[client]["status"] = "online"
                     clients["update_active_clients"].append(client)
@@ -366,9 +445,15 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
                 clients["update_offline_clients"].append(client)
         # Update statestore with client status
         if len(clients["update_active_clients"]) > 0:
-            self.statestore.update_client_status(clients["update_active_clients"], "online")
+            for client in clients["update_active_clients"]:
+                client_to_update = self.db.client_store.get(client)
+                client_to_update.status = "online"
+                self.db.client_store.update(client_to_update)
         if len(clients["update_offline_clients"]) > 0:
-            self.statestore.update_client_status(clients["update_offline_clients"], "offline")
+            for client in clients["update_offline_clients"]:
+                client_to_update = self.db.client_store.get(client)
+                client_to_update.status = "offline"
+                self.db.client_store.update(client_to_update)
 
         return clients["active_clients"]
 
@@ -398,10 +483,12 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
     def _send_status(self, status):
         """Report a status to backend db.
 
-        :param status: the status to report
+        :param status: the status message to report
         :type status: :class:`fedn.network.grpc.fedn_pb2.Status`
         """
-        self.statestore.report_status(status)
+        data = MessageToDict(status, preserving_proto_field_name=True)
+        status = StatusDTO().populate_with(data)
+        self.db.status_store.add(status)
 
     def _flush_model_update_queue(self):
         """Clear the model update queue (aggregator).
@@ -464,7 +551,6 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         logger.debug("grpc.Combiner.SetAggregator: Called")
         for parameter in control.parameter:
             aggregator = parameter.value
-
         status = self.round_handler.set_aggregator(aggregator)
 
         response = fedn.ControlResponse()
@@ -472,7 +558,27 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
             response.message = "Success"
         else:
             response.message = "Failed"
+        return response
 
+    def SetServerFunctions(self, control: fedn.ControlRequest, context):
+        """Set a function provider.
+
+        :param control: the control request
+        :type control: :class:`fedn.network.grpc.fedn_pb2.ControlRequest`
+        :param context: the context (unused)
+        :type context: :class:`grpc._server._Context`
+        :return: the control response
+        :rtype: :class:`fedn.network.grpc.fedn_pb2.ControlResponse`
+        """
+        logger.debug("grpc.Combiner.SetServerFunctions: Called")
+        for parameter in control.parameter:
+            server_functions = parameter.value
+
+        self.round_handler.set_server_functions(server_functions)
+
+        response = fedn.ControlResponse()
+        response.message = "Success"
+        logger.info(f"set function provider response {response}")
         return response
 
     def FlushAggregationQueue(self, control: fedn.ControlRequest, context):
@@ -552,7 +658,7 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
             logger.info("grpc.Combiner.ListActiveClients: Number active clients: {}".format(nr_active_clients))
 
         for client in active_clients:
-            clients.client.append(fedn.Client(name=client, role=fedn.WORKER))
+            clients.client.append(fedn.Client(name=client, role=fedn.CLIENT, client_id=client))
         return clients
 
     def AcceptingClients(self, request: fedn.ConnectionRequest, context):
@@ -624,12 +730,13 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
             metadata = dict(metadata)
             logger.info("grpc.Combiner.TaskStream: Client connected: {}\n".format(metadata["client"]))
 
-        status = fedn.Status(status="Client {} connecting to TaskStream.".format(client.name))
-        status.log_level = fedn.Status.INFO
+        status = fedn.Status(status="Client {} connecting to TaskStream.".format(client.name), log_level=fedn.LogLevel.INFO, type=fedn.StatusType.NETWORK)
+        logger.info("Client {} connecting to TaskStream.".format(client.name))
         status.timestamp.GetCurrentTime()
 
         self.__whoami(status.sender, self)
 
+        # Subscribe client, this also adds the client to self.clients
         self._subscribe_client_to_queue(client, fedn.Queue.TASK_QUEUE)
         q = self.__get_queue(client, fedn.Queue.TASK_QUEUE)
 
@@ -637,7 +744,19 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
 
         # Set client status to online
         self.clients[client.client_id]["status"] = "online"
-        self.statestore.set_client({"name": client.name, "status": "online", "client_id": client.client_id, "last_seen": datetime.now()})
+        try:
+            # If the client is already in the client store, update the status
+            client_to_update = self.db.client_store.get(client.client_id)
+            if client_to_update is not None:
+                client_to_update.status = "online"
+                client_to_update.last_seen = datetime.now()
+                self.db.client_store.update(client_to_update)
+            else:
+                new_client = ClientDTO(client_id=client.client_id, name=client.name, status="online", last_seen=datetime.now(), combiner=self.id)
+                self.db.client_store.add(new_client)
+
+        except Exception as e:
+            logger.error(f"Failed to update client status: {str(e)}")
 
         # Keep track of the time context has been active
         start_time = time.time()
@@ -653,6 +772,13 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
                 pass
             except Exception as e:
                 logger.error("Error in ModelUpdateRequestStream: {}".format(e))
+        logger.warning("Client {} disconnected from TaskStream".format(client.name))
+        status = fedn.Status(status="Client {} disconnected from TaskStream.".format(client.name))
+        status.log_level = fedn.LogLevel.INFO
+        status.type = fedn.StatusType.NETWORK
+        status.timestamp.GetCurrentTime()
+        self.__whoami(status.sender, self)
+        self._send_status(status)
 
     def SendModelUpdate(self, request, context):
         """Send a model update response.
@@ -664,7 +790,7 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         :return: the response
         :rtype: :class:`fedn.network.grpc.fedn_pb2.Response`
         """
-        self.round_handler.aggregator.on_model_update(request)
+        self.round_handler.update_handler.on_model_update(request)
 
         response = fedn.Response()
         response.response = "RECEIVED ModelUpdate {} from client  {}".format(response, response.sender.name)
@@ -676,7 +802,13 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         :param validation: the model validation
         :type validation: :class:`fedn.network.grpc.fedn_pb2.ModelValidation`
         """
-        self.statestore.report_validation(validation)
+        data = MessageToDict(validation, preserving_proto_field_name=True)
+        validationdto = ValidationDTO(**data)
+        try:
+            result = self.db.validation_store.add(validationdto)
+            logger.info("Model validation registered: {}".format(result))
+        except Exception as e:
+            logger.error(f"Failed to register model validation: {e}")
 
     def SendModelValidation(self, request, context):
         """Send a model validation response.
@@ -690,12 +822,131 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         """
         logger.info("Recieved ModelValidation from {}".format(request.sender.name))
 
-        self.register_model_validation(request)
+        data = MessageToDict(request, preserving_proto_field_name=True)
+        validationdto = ValidationDTO(**data)
+        self.db.validation_store.add(validationdto)
 
         response = fedn.Response()
         response.response = "RECEIVED ModelValidation {} from client  {}".format(response, response.sender.name)
         return response
 
+    def SendModelPrediction(self, request, context):
+        """Send a model prediction response.
+
+        :param request: the request
+        :type request: :class:`fedn.network.grpc.fedn_pb2.ModelPrediction`
+        :param context: the context
+        :type context: :class:`grpc._server._Context`
+        :return: the response
+        :rtype: :class:`fedn.network.grpc.fedn_pb2.Response`
+        """
+        logger.info("Recieved ModelPrediction from {}".format(request.sender.name))
+
+        data = MessageToDict(request, preserving_proto_field_name=True)
+
+        prediction = PredictionDTO(**data)
+        self.db.prediction_store.add(prediction)
+
+        response = fedn.Response()
+        response.response = "RECEIVED ModelPrediction {} from client  {}".format(response, response.sender.name)
+        return response
+
+    def SendBackwardCompletion(self, request, context):
+        """Send a backward completion response.
+
+        :param request: the request
+        :type request: :class:`fedn.network.grpc.fedn_pb2.BackwardCompletion`
+        :param context: the context
+        :type context: :class:`grpc._server._Context`
+        :return: the response
+        :rtype: :class:`fedn.network.grpc.fedn_pb2.Response`
+        """
+        logger.info("Received BackwardCompletion from {}".format(request.sender.name))
+
+        # Add completion to the queue
+        self.round_handler.update_handler.backward_completions.put(request)
+
+        # Create and send status message for backward completion
+        status = fedn.Status()
+        status.timestamp.GetCurrentTime()
+        status.sender.name = request.sender.name
+        status.sender.role = request.sender.role
+        status.sender.client_id = request.sender.client_id
+        status.status = "finished_backward"
+        status.type = fedn.StatusType.BACKWARD
+        status.session_id = request.session_id
+
+        logger.info(f"Creating status message with session_id: {request.session_id}")
+        self._send_status(status)
+        logger.info("Status message sent to MongoDB")
+
+        ###########
+
+        response = fedn.Response()
+        response.response = "RECEIVED BackwardCompletion from client {}".format(request.sender.name)
+        return response
+
+    def SendModelMetric(self, request, context):
+        """Send a model metric response.
+
+        :param request: the request
+        :type request: :class:`fedn.network.grpc.fedn_pb2.ModelMetric`
+        :param context: the context
+        :type context: :class:`grpc._server._Context`
+        :return: the response
+        :rtype: :class:`fedn.network.grpc.fedn_pb2.Response`
+        """
+        logger.debug("Received ModelMetric from {}".format(request.sender.name))
+        metric_msg = MessageToDict(request, preserving_proto_field_name=True)
+        metrics = metric_msg.pop("metrics")
+
+        for metric in metrics:
+            new_metric = MetricDTO(**metric, **metric_msg)
+            try:
+                self.db.metric_store.add(new_metric)
+            except Exception as e:
+                logger.error(f"Failed to register model metric: {e}")
+
+        return fedn.Response()
+
+    def SendAttributeMessage(self, request, context):
+        """Send a model attribute response.
+
+        :param request: the request
+        :type request: :class:`fedn.network.grpc.fedn_pb2.AttributeMessage`
+        """
+        logger.debug("Received Attributes from {}".format(request.sender.name))
+        attribute_msg = MessageToDict(request, preserving_proto_field_name=True)
+        attributes = attribute_msg.pop("attributes")
+
+        for attribute in attributes:
+            new_attribute = AttributeDTO(**attribute, **attribute_msg)
+            try:
+                self.db.attribute_store.add(new_attribute)
+            except Exception as e:
+                logger.error(f"Failed to register model attribute: {e}")
+
+        return fedn.Response()
+
+    def SendTelemetryMessage(self, request, context):
+        """Send a telemetry message.
+
+        :param request: the request
+        :type request: :class:`fedn.network.grpc.fedn_pb2.TelemetryMessage`
+        """
+        logger.debug("Received Telemetry from {}".format(request.sender.name))
+        telemetry_msg = MessageToDict(request, preserving_proto_field_name=True)
+        telemetries = telemetry_msg.pop("telemetries")
+        for telemetry in telemetries:
+            telemetry = {"value": 0.0, **telemetry}
+            new_telemetry = TelemetryDTO(**telemetry, **telemetry_msg)
+            try:
+                self.db.telemetry_store.add(new_telemetry)
+            except Exception as e:
+                logger.error(f"Failed to register telemetry: {e}")
+
+        return fedn.Response()
+
     ####################################################################################################################
 
     def run(self):
@@ -707,3 +958,4 @@ class Combiner(rpc.CombinerServicer, rpc.ReducerServicer, rpc.ConnectorServicer,
         except (KeyboardInterrupt, SystemExit):
             pass
         self.server.stop()
+        self.server.stop()
