@@ -1,8 +1,6 @@
 import base64
 import copy
 import json
-import time
-from io import BytesIO
 
 import grpc
 
@@ -125,13 +123,6 @@ class CombinerInterface:
             "key": None,
             "config": self.config,
         }
-
-        if self.certificate:
-            cert_b64 = base64.b64encode(self.certificate)
-            key_b64 = base64.b64encode(self.key)
-            data["certificate"] = str(cert_b64).split("'")[1]
-            data["key"] = str(key_b64).split("'")[1]
-
         return data
 
     def to_json(self):
@@ -203,6 +194,28 @@ class CombinerInterface:
             else:
                 raise
 
+    def set_server_functions(self, server_functions):
+        """Set the function provider module.
+
+        :param function provider: Stringified function provider code.
+        :type config: str
+        """
+        channel = Channel(self.address, self.port, self.certificate).get_channel()
+        control = rpc.ControlStub(channel)
+
+        request = fedn.ControlRequest()
+        p = request.parameter.add()
+        p.key = "server_functions"
+        p.value = server_functions
+
+        try:
+            control.SetServerFunctions(request)
+        except grpc.RpcError as e:
+            if e.code() == grpc.StatusCode.UNAVAILABLE:
+                raise CombinerUnavailableError
+            else:
+                raise
+
     def submit(self, config: RoundConfig):
         """Submit a compute plan to the combiner.
 
@@ -230,39 +243,6 @@ class CombinerInterface:
 
         return response
 
-    def get_model(self, id, timeout=10):
-        """Download a model from the combiner server.
-
-        :param id: The model id.
-        :type id: str
-        :return: A file-like object containing the model.
-        :rtype: :class:`io.BytesIO`, None if the model is not available.
-        """
-        channel = Channel(self.address, self.port, self.certificate).get_channel()
-        modelservice = rpc.ModelServiceStub(channel)
-
-        data = BytesIO()
-        data.seek(0, 0)
-
-        time_start = time.time()
-
-        request = fedn.ModelRequest(id=id)
-        request.sender.name = self.name
-        request.sender.role = fedn.WORKER
-
-        parts = modelservice.Download(request)
-        for part in parts:
-            if part.status == fedn.ModelStatus.IN_PROGRESS:
-                data.write(part.data)
-            if part.status == fedn.ModelStatus.OK:
-                return data
-            if part.status == fedn.ModelStatus.FAILED:
-                return None
-            if part.status == fedn.ModelStatus.UNKNOWN:
-                if time.time() - time_start > timeout:
-                    return None
-                continue
-
     def allowing_clients(self):
         """Check if the combiner is allowing additional client connections.
 
