@@ -6,16 +6,14 @@ class Config(ABC):
 
 
 class ReducerConfig(Config):
-    """ Configuration for the Reducer component. """
+    """Configuration for the Reducer component."""
+
     compute_bundle_dir = None
     models_dir = None
 
     initial_model = None
 
-    storage_backend = {
-        'type': 's3', 'settings':
-            {'bucket': 'models'}
-    }
+    storage_backend = {"type": "s3", "settings": {"bucket": "models"}}
 
     def __init__(self):
         pass
