@@ -1,16 +1,21 @@
 import copy
 import datetime
 import time
-import uuid
+from typing import Optional
 
 from tenacity import retry, retry_if_exception_type, stop_after_delay, wait_random
 
 from fedn.common.log_config import logger
 from fedn.network.combiner.interfaces import CombinerUnavailableError
-from fedn.network.combiner.modelservice import load_model_from_BytesIO
+from fedn.network.combiner.modelservice import load_model_from_bytes
 from fedn.network.combiner.roundhandler import RoundConfig
 from fedn.network.controller.controlbase import ControlBase
 from fedn.network.state import ReducerState
+from fedn.network.storage.dbconnection import DatabaseConnection
+from fedn.network.storage.s3.repository import Repository
+from fedn.network.storage.statestore.stores.dto.run import RunDTO
+from fedn.network.storage.statestore.stores.dto.session import SessionConfigDTO
+from fedn.network.storage.statestore.stores.shared import SortOrder
 
 
 class UnsupportedStorageBackend(Exception):
@@ -82,56 +87,126 @@ class SessionTerminatedException(Exception):
 
 
 class Control(ControlBase):
-    """Controller, implementing the overall global training, validation and inference logic.
+    """Controller, implementing the overall global training, validation and prediction logic.
 
     :param statestore: A StateStorage instance.
     :type statestore: class: `fedn.network.statestorebase.StateStorageBase`
     """
 
-    def __init__(self, statestore):
+    _instance: "Control"
+
+    def __init__(
+        self,
+        network_id: str,
+        repository: Repository,
+        db: DatabaseConnection,
+    ):
         """Constructor method."""
-        super().__init__(statestore)
+        super().__init__(network_id, repository, db)
         self.name = "DefaultControl"
 
-    def start_session(self, session_id: str, rounds: int, round_timeout: int) -> None:
+    @classmethod
+    def instance(cls) -> "Control":
+        """Get the singleton instance of the Control class."""
+        if Control._instance is None:
+            raise Exception("Control instance not initialized")
+        return Control._instance
+
+    @classmethod
+    def create_instance(cls, network_id: str, repository: Repository, db: DatabaseConnection) -> "Control":
+        """Create a singleton instance of the Control class.
+
+        :param network_id: The network ID.
+        :type network_id: str
+        :param repository: The repository instance.
+        :type repository: Repository
+        :param db: The database connection instance.
+        :type db: DatabaseConnection
+        :return: The Control instance.
+        :rtype: Control
+        """
+        cls._instance = cls(network_id, repository, db)
+        return cls._instance
+
+    def _get_active_model_id(self, session_id: str) -> Optional[str]:
+        """Get the active model for a session.
+
+        :param session_id: The session ID.
+        :type session_id: str
+        :return: The active model ID.
+        :rtype: str
+        """
+        last_model_of_session = self.db.model_store.list(1, 0, "committed_at", SortOrder.DESCENDING, session_id=session_id)
+        if len(last_model_of_session) > 0:
+            return last_model_of_session[0].model_id
+
+        # if no model is found for the session, get the last model in the model chain
+        last_model = self.db.model_store.list(1, 0, "committed_at", SortOrder.DESCENDING)
+        if len(last_model) > 0:
+            return last_model[0].model_id
+
+        return None
+
+    def start_session(
+        self, session_id: str, rounds: int, round_timeout: int, model_name_prefix: Optional[str] = None, client_ids: Optional[list[str]] = None
+    ) -> None:
         if self._state == ReducerState.instructing:
             logger.info("Controller already in INSTRUCTING state. A session is in progress.")
             return
 
-        if not self.statestore.get_latest_model():
-            logger.warning("No model in model chain, please provide a seed model!")
+        try:
+            active_model_id = self._get_active_model_id(session_id)
+            if not active_model_id or active_model_id in ["", " "]:
+                logger.warning("No model in model chain, please provide a seed model!")
+                return
+        except Exception:
+            logger.error("Failed to get latest model of session and model chain.")
             return
 
         self._state = ReducerState.instructing
 
-        session = self.statestore.get_session(session_id)
+        session = self.db.session_store.get(session_id)
 
         if not session:
             logger.error("Session not found.")
             return
 
-        session_config = session["session_config"]
+        session_config = session.session_config
 
-        if not session_config or not isinstance(session_config, dict):
+        if not session_config:
             logger.error("Session not properly configured.")
             return
 
         if round_timeout is not None:
-            session_config["round_timeout"] = round_timeout
+            session_config.round_timeout = round_timeout
 
         self._state = ReducerState.monitoring
 
-        last_round = int(self.get_latest_round_id())
+        last_round = self.get_latest_round_id()
 
-        aggregator = session_config["aggregator"]
-
-        session_config["session_id"] = session_id
+        aggregator = session_config.aggregator
 
         for combiner in self.network.get_combiners():
             combiner.set_aggregator(aggregator)
+            if session_config.server_functions is not None:
+                combiner.set_server_functions(session_config.server_functions)
 
         self.set_session_status(session_id, "Started")
 
+        training_run_obj = RunDTO()
+        training_run_obj.session_id = session_id
+        training_run_obj.model_id = active_model_id
+        training_run_obj.round_timeout = session_config.round_timeout
+        training_run_obj.rounds = rounds
+
+        training_run_obj = self.db.run_store.add(training_run_obj)
+
+        count_models_of_session = 0
+
+        if model_name_prefix is not None:
+            count_models_of_session = self.db.model_store.count(session_id=session_id)
+            count_models_of_session += 1
+
         for round in range(1, rounds + 1):
             if last_round:
                 current_round = last_round + round
@@ -141,81 +216,39 @@ class Control(ControlBase):
             try:
                 if self.get_session_status(session_id) == "Terminated":
                     logger.info("Session terminated.")
+                    training_run_obj.completed_at = datetime.datetime.now()
+                    training_run_obj.completed_at_model_id = self._get_active_model_id(session_id)
+                    self.db.run_store.update(training_run_obj)
                     break
-                _, round_data = self.round(session_config, str(current_round))
+                _, round_data = self.round(
+                    session_config=session_config,
+                    round_id=str(current_round),
+                    session_id=session_id,
+                    model_name=f"{model_name_prefix}_{count_models_of_session}" if model_name_prefix else None,
+                    client_ids=client_ids,
+                )
+                count_models_of_session += 1
+                logger.info("Round completed with status {}".format(round_data.status))
             except TypeError as e:
                 logger.error("Failed to execute round: {0}".format(e))
 
-            logger.info("Round completed with status {}".format(round_data["status"]))
-
-            session_config["model_id"] = self.statestore.get_latest_model()
+            session_config.model_id = self._get_active_model_id(session_id)
 
         if self.get_session_status(session_id) == "Started":
             self.set_session_status(session_id, "Finished")
+            training_run_obj.completed_at = datetime.datetime.now()
+            training_run_obj.completed_at_model_id = self._get_active_model_id(session_id)
+            self.db.run_store.update(training_run_obj)
+            logger.info("Session finished.")
         self._state = ReducerState.idle
 
-        self.set_session_config(session_id, session_config)
+        self.set_session_config(session_id, session_config.to_dict())
 
-    def session(self, config: RoundConfig) -> None:
-        """Execute a new training session. A session consists of one
-            or several global rounds. All rounds in the same session
-            have the same round_config.
-
-        :param config: The session config.
-        :type config: dict
-
-        """
-        if self._state == ReducerState.instructing:
-            logger.info("Controller already in INSTRUCTING state. A session is in progress.")
-            return
-
-        if not self.statestore.get_latest_model():
-            logger.warning("No model in model chain, please provide a seed model!")
-            return
-
-        self._state = ReducerState.instructing
-        config["committed_at"] = datetime.datetime.now()
-
-        self.create_session(config)
-
-        self._state = ReducerState.monitoring
-
-        last_round = int(self.get_latest_round_id())
-
-        for combiner in self.network.get_combiners():
-            combiner.set_aggregator(config["aggregator"])
-
-        self.set_session_status(config["session_id"], "Started")
-        # Execute the rounds in this session
-        for round in range(1, int(config["rounds"] + 1)):
-            # Increment the round number
-            if last_round:
-                current_round = last_round + round
-            else:
-                current_round = round
-
-            try:
-                if self.get_session_status(config["session_id"]) == "Terminated":
-                    logger.info("Session terminated.")
-                    break
-                _, round_data = self.round(config, str(current_round))
-            except TypeError as e:
-                logger.error("Failed to execute round: {0}".format(e))
-
-            logger.info("Round completed with status {}".format(round_data["status"]))
-
-            config["model_id"] = self.statestore.get_latest_model()
-
-        # TODO: Report completion of session
-        if self.get_session_status(config["session_id"]) == "Started":
-            self.set_session_status(config["session_id"], "Finished")
-        self._state = ReducerState.idle
-
-    def inference_session(self, config: RoundConfig) -> None:
-        """Execute a new inference session.
+    def prediction_session(self, config: RoundConfig) -> None:
+        """Execute a new prediction session.
 
         :param config: The round config.
-        :type config: InferenceConfig
+        :type config: PredictionConfig
         :return: None
         """
         if self._state == ReducerState.instructing:
@@ -223,14 +256,14 @@ class Control(ControlBase):
             return
 
         if len(self.network.get_combiners()) < 1:
-            logger.warning("Inference round cannot start, no combiners connected!")
+            logger.warning("Prediction round cannot start, no combiners connected!")
             return
 
         if "model_id" not in config.keys():
-            config["model_id"] = self.statestore.get_latest_model()
+            config["model_id"] = self.db.model_store.get_active()
 
         config["committed_at"] = datetime.datetime.now()
-        config["task"] = "inference"
+        config["task"] = "prediction"
         config["rounds"] = str(1)
         config["clients_required"] = 1
 
@@ -240,12 +273,83 @@ class Control(ControlBase):
         round_start = self.evaluate_round_start_policy(participating_combiners)
 
         if round_start:
-            logger.info("Inference round start policy met, {} participating combiners.".format(len(participating_combiners)))
+            logger.info("Prediction round start policy met, {} participating combiners.".format(len(participating_combiners)))
             for combiner, _ in participating_combiners:
                 combiner.submit(config)
-                logger.info("Inference round submitted to combiner {}".format(combiner))
+                logger.info("Prediction round submitted to combiner {}".format(combiner))
 
-    def round(self, session_config: RoundConfig, round_id: str):
+    def splitlearning_session(self, session_id: str, rounds: int, round_timeout: int) -> None:
+        """Execute a split learning session.
+
+        :param session_id: The session id.
+        :type session_id: str
+        :param rounds: The number of rounds.
+        :type rounds: int
+        :param round_timeout: The round timeout.
+        :type round_timeout: int
+        """
+        logger.info("Starting split learning session.")
+
+        if self._state == ReducerState.instructing:
+            logger.info("Controller already in INSTRUCTING state. A session is in progress.")
+            return
+
+        self._state = ReducerState.instructing
+
+        session = self.db.session_store.get(session_id)
+
+        if not session:
+            logger.error("Session not found.")
+            return
+
+        session_config = session.session_config
+
+        if not session_config:
+            logger.error("Splitlearning session not properly configured.")
+            return
+
+        if round_timeout is not None:
+            session_config.round_timeout = round_timeout
+
+        self._state = ReducerState.monitoring
+
+        last_round = self.get_latest_round_id()
+
+        for combiner in self.network.get_combiners():
+            combiner.set_aggregator(session_config.aggregator)
+
+        session_config.session_id = session_id
+
+        self.set_session_status(session_id, "Started")
+
+        # Execute the rounds in this session
+        for round in range(1, rounds + 1):
+            if last_round:
+                current_round = last_round + round
+            else:
+                current_round = round
+
+            try:
+                if self.get_session_status(session_config.session_id) == "Terminated":
+                    logger.info("Session terminated.")
+                    break
+                _, round_obj = self.splitlearning_round(session_config, str(current_round), session_id)
+                if round_obj:
+                    logger.info("Split learning round completed with status {}".format(round_obj.status))
+                else:
+                    logger.error("Split learning round failed - no round data returned")
+            except TypeError as e:
+                logger.error("Failed to execute split learning round: {0}".format(e))
+
+        if self.get_session_status(session_config.session_id) == "Started":
+            self.set_session_status(session_config.session_id, "Finished")
+        self._state = ReducerState.idle
+
+        self.set_session_config(session_id, session_config.to_dict())
+
+    def round(
+        self, session_config: SessionConfigDTO, round_id: str, session_id: str, model_name: Optional[str] = None, client_ids: Optional[list[str]] = None
+    ) -> tuple:
         """Execute one global round.
 
         : param session_config: The session config.
@@ -254,21 +358,23 @@ class Control(ControlBase):
         : type round_id: str
 
         """
-        session_id = session_config["session_id"]
         self.create_round({"round_id": round_id, "status": "Pending"})
 
         if len(self.network.get_combiners()) < 1:
             logger.warning("Round cannot start, no combiners connected!")
             self.set_round_status(round_id, "Failed")
-            return None, self.statestore.get_round(round_id)
+            return None, self.db.round_store.get(round_id)
 
         # Assemble round config for this global round
-        round_config = copy.deepcopy(session_config)
+        round_config: RoundConfig = session_config.to_dict()
         round_config["rounds"] = 1
         round_config["round_id"] = round_id
         round_config["task"] = "training"
         round_config["session_id"] = session_id
 
+        if client_ids and len(client_ids) > 0:
+            round_config["selected_clients"] = client_ids
+
         self.set_round_config(round_id, round_config)
 
         # Get combiners that are able to participate in the round, given round_config
@@ -282,7 +388,7 @@ class Control(ControlBase):
         else:
             logger.warning("Round start policy not met, skipping round!")
             self.set_round_status(round_id, "Failed")
-            return None, self.statestore.get_round(round_id)
+            return None, self.db.round_store.get(round_id)
 
         # Ask participating combiners to coordinate model updates
         _ = self.request_model_updates(participating_combiners)
@@ -296,21 +402,21 @@ class Control(ControlBase):
 
         @retry(
             wait=wait_random(min=1.0, max=2.0),
-            stop=stop_after_delay(session_config["round_timeout"]),
+            stop=stop_after_delay(session_config.round_timeout),
             retry_error_callback=do_if_round_times_out,
             retry=retry_if_exception_type(CombinersNotDoneException),
         )
         def combiners_done():
-            round = self.statestore.get_round(round_id)
+            round = self.db.round_store.get(round_id)
             session_status = self.get_session_status(session_id)
             if session_status == "Terminated":
                 self.set_round_status(round_id, "Terminated")
                 return False
-            if "combiners" not in round:
+            if len(round.combiners) < 1:
                 logger.info("Waiting for combiners to update model...")
                 raise CombinersNotDoneException("Combiners have not yet reported.")
 
-            if len(round["combiners"]) < len(participating_combiners):
+            if len(round.combiners) < len(participating_combiners):
                 logger.info("Waiting for combiners to update model...")
                 raise CombinersNotDoneException("All combiners have not yet reported.")
 
@@ -318,63 +424,71 @@ class Control(ControlBase):
 
         combiners_are_done = combiners_done()
         if not combiners_are_done:
-            return None, self.statestore.get_round(round_id)
+            return None, self.db.round_store.get(round_id)
 
         # Due to the distributed nature of the computation, there might be a
         # delay before combiners have reported the round data to the db,
         # so we need some robustness here.
         @retry(wait=wait_random(min=0.1, max=1.0), retry=retry_if_exception_type(KeyError))
         def check_combiners_done_reporting():
-            round = self.statestore.get_round(round_id)
-            combiners = round["combiners"]
-            return combiners
+            round = self.db.round_store.get(round_id)
+            if len(round.combiners) != len(participating_combiners):
+                raise KeyError("Combiners have not yet reported.")
 
-        _ = check_combiners_done_reporting()
+        check_combiners_done_reporting()
 
-        round = self.statestore.get_round(round_id)
-        round_valid = self.evaluate_round_validity_policy(round)
+        round = self.db.round_store.get(round_id)
+        round_valid = self.evaluate_round_validity_policy(round.to_dict())
         if not round_valid:
             logger.error("Round failed. Invalid - evaluate_round_validity_policy: False")
             self.set_round_status(round_id, "Failed")
-            return None, self.statestore.get_round(round_id)
+            return None, self.db.round_store.get(round_id)
 
         logger.info("Reducing combiner level models...")
         # Reduce combiner models into a new global model
         round_data = {}
         try:
-            round = self.statestore.get_round(round_id)
-            model, data = self.reduce(round["combiners"])
+            round = self.db.round_store.get(round_id)
+            model, data = self.reduce(round.combiners.to_dict())
             round_data["reduce"] = data
             logger.info("Done reducing models from combiners!")
         except Exception as e:
             logger.error("Failed to reduce models from combiners, reason: {}".format(e))
             self.set_round_status(round_id, "Failed")
-            return None, self.statestore.get_round(round_id)
+            return None, self.db.round_store.get(round_id)
 
         # Commit the new global model to the model trail
+        model_id: Optional[str] = None
         if model is not None:
             logger.info("Committing global model to model trail...")
             tic = time.time()
-            model_id = uuid.uuid4()
-            session_id = session_config["session_id"] if "session_id" in session_config else None
-            self.commit(model_id, model, session_id)
+            model_id = self.commit(model=model, session_id=session_id, name=model_name)
             round_data["time_commit"] = time.time() - tic
             logger.info("Done committing global model to model trail.")
         else:
             logger.error("Failed to commit model to global model trail.")
             self.set_round_status(round_id, "Failed")
-            return None, self.statestore.get_round(round_id)
+            return None, self.db.round_store.get(round_id)
 
         self.set_round_status(round_id, "Success")
 
         # 4. Trigger participating combiner nodes to execute a validation round for the current model
-        validate = session_config["validate"]
-        if validate:
-            combiner_config = copy.deepcopy(session_config)
+        if session_config.validate:
+            combiner_config = session_config.to_dict()
             combiner_config["round_id"] = round_id
-            combiner_config["model_id"] = self.statestore.get_latest_model()
+            combiner_config["model_id"] = model_id
             combiner_config["task"] = "validation"
-            combiner_config["helper_type"] = self.statestore.get_helper()
+            combiner_config["session_id"] = session_id
+
+            helper_type: str = None
+
+            try:
+                active_package = self.db.package_store.get_active()
+                helper_type = active_package.helper
+            except Exception:
+                logger.error("Failed to get active helper")
+
+            combiner_config["helper_type"] = helper_type
 
             validating_combiners = self.get_participating_combiners(combiner_config)
 
@@ -388,7 +502,148 @@ class Control(ControlBase):
 
         self.set_round_data(round_id, round_data)
         self.set_round_status(round_id, "Finished")
-        return model_id, self.statestore.get_round(round_id)
+        return model_id, self.db.round_store.get(round_id)
+
+    def splitlearning_round(self, session_config: SessionConfigDTO, round_id: str, session_id: str):
+        """Execute one global split learning round
+
+        :param session_config: The session config
+        :type session_config: SessionConfigDTO
+        :param round_id: The round id
+        :type round_id: str
+        :param session_id: The session id
+        :type session_id: str
+        """
+        # session_id = session_config.session_id
+        self.create_round({"round_id": round_id, "status": "Pending"})
+
+        if len(self.network.get_combiners()) < 1:
+            logger.warning("Round cannot start, no combiners connected!")
+            self.set_round_status(round_id, "Failed")
+            return None, self.db.round_store.get(round_id)
+
+        # 1) FORWARD PASS - specified through "task": "forward"
+        forward_config = session_config.to_dict()
+        forward_config.update({"rounds": 1, "round_id": round_id, "task": "forward", "is_sl_inference": False, "session_id": session_id})
+
+        self.set_round_config(round_id, forward_config)
+
+        participating_combiners = self.get_participating_combiners(forward_config)
+
+        if not self.evaluate_round_start_policy(participating_combiners):
+            logger.warning("Round start policy not met, skipping round!")
+            self.set_round_status(round_id, "Failed")
+            return None, self.db.round_store.get(round_id)
+
+        logger.info("CONTROLLER: Requesting forward pass")
+        # Request forward pass using existing method
+        _ = self.request_model_updates(participating_combiners)
+
+        # Wait until participating combiners have produced an updated global model,
+        # or round times out.
+        def do_if_round_times_out(result):
+            logger.warning("Round timed out!")
+            return True
+
+        @retry(
+            wait=wait_random(min=1.0, max=2.0),
+            stop=stop_after_delay(session_config.round_timeout),
+            retry_error_callback=do_if_round_times_out,
+            retry=retry_if_exception_type(CombinersNotDoneException),
+        )
+        def combiners_done():
+            round = self.db.round_store.get(round_id)
+            session_status = self.get_session_status(session_id)
+            if session_status == "Terminated":
+                self.set_round_status(round_id, "Terminated")
+                return False
+            if len(round.combiners) < 1:
+                logger.info("Waiting for combiners to update model...")
+                raise CombinersNotDoneException("Combiners have not yet reported.")
+
+            if len(round.combiners) < len(participating_combiners):
+                logger.info("Waiting for combiners to update model...")
+                raise CombinersNotDoneException("All combiners have not yet reported.")
+
+            return True
+
+        combiners_are_done = combiners_done()
+        if not combiners_are_done:
+            return None, self.db.round_store.get(round_id)
+
+        # Due to the distributed nature of the computation, there might be a
+        # delay before combiners have reported the round data to the db,
+        # so we need some robustness here.
+        @retry(wait=wait_random(min=0.1, max=1.0), retry=retry_if_exception_type(KeyError))
+        def check_combiners_done_reporting():
+            round = self.db.round_store.get(round_id)
+            if len(round.combiners) != len(participating_combiners):
+                raise KeyError("Combiners have not yet reported.")
+
+        check_combiners_done_reporting()
+
+        logger.info("CONTROLLER: Forward pass completed.")
+
+        # NOTE: Only works for one combiner
+        # get model id and send it to backward pass
+        round = self.db.round_store.get(round_id)
+        round = round.to_dict()
+        for combiner in round["combiners"]:
+            try:
+                model_id = combiner["model_id"]
+            except KeyError:
+                logger.error("Forward pass failed - no model_id in combiner response")
+                self.set_round_status(round_id, "Failed")
+                return None, self.db.round_store.get(round_id)
+
+        if model_id is None:
+            logger.error("Forward pass failed - no model_id in combiner response")
+            self.set_round_status(round_id, "Failed")
+            return None, self.db.round_store.get(round_id)
+
+        logger.info("CONTROLLER: starting backward pass with model/gradient id: {}".format(model_id))
+
+        # 2) BACKWARD PASS
+        try:
+            backward_config = session_config.to_dict()
+            backward_config.update({"rounds": 1, "round_id": round_id, "task": "backward", "session_id": session_id, "model_id": model_id})
+
+            participating_combiners = [(combiner, backward_config) for combiner, _ in participating_combiners]
+            result = self.request_model_updates(participating_combiners)
+
+            if not result:
+                logger.error("Backward pass failed - no result from model updates")
+                self.set_round_status(round_id, "Failed")
+                return None, self.db.round_store.get(round_id)
+
+            logger.info("CONTROLLER: Backward pass completed successfully")
+            self.set_round_status(round_id, "Success")
+
+        except Exception as e:
+            logger.error(f"Backward pass failed with error: {e}")
+            self.set_round_status(round_id, "Failed")
+            return None, self.db.round_store.get(round_id)
+
+        # 3) Validation
+        validate = session_config.validate
+        if validate:
+            logger.info("CONTROLLER: Starting Split Learning Validation round")
+            validate_config = session_config.to_dict()
+            validate_config.update({"rounds": 1, "round_id": round_id, "task": "forward", "is_sl_inference": True, "session_id": session_id})
+            validating_combiners = [(combiner, validate_config) for combiner, _ in participating_combiners]
+
+            # Submit validation requests
+            for combiner, config in validating_combiners:
+                try:
+                    logger.info("Submitting validation for split learning to combiner {}".format(combiner))
+                    combiner.submit(config)
+                except CombinerUnavailableError:
+                    self._handle_unavailable_combiner(combiner)
+                    pass
+            logger.info("Controller: Split Learning Validation completed")
+
+        self.set_round_status(round_id, "Finished")
+        return None, self.db.round_store.get(round_id)
 
     def reduce(self, combiners):
         """Combine updated models from Combiner nodes into one global model.
@@ -412,7 +667,7 @@ class Control(ControlBase):
 
             try:
                 tic = time.time()
-                data = self.model_repository.get_model(model_id)
+                data = self.repository.get_model(model_id)
                 meta["time_fetch_model"] += time.time() - tic
             except Exception as e:
                 logger.error("Failed to fetch model from model repository {}: {}".format(name, e))
@@ -422,26 +677,28 @@ class Control(ControlBase):
                 try:
                     tic = time.time()
                     helper = self.get_helper()
-                    model_next = load_model_from_BytesIO(data, helper)
+                    model_next = load_model_from_bytes(data, helper)
                     meta["time_load_model"] += time.time() - tic
                     tic = time.time()
                     model = helper.increment_average(model, model_next, 1.0, i)
                     meta["time_aggregate_model"] += time.time() - tic
                 except Exception:
                     tic = time.time()
-                    model = load_model_from_BytesIO(data, helper)
+                    model = load_model_from_bytes(data, helper)
                     meta["time_aggregate_model"] += time.time() - tic
                 i = i + 1
 
-            self.model_repository.delete_model(model_id)
+            self.repository.delete_model(model_id)
 
         return model, meta
 
-    def infer_instruct(self, config):
-        """Main entrypoint for executing the inference compute plan.
+    def predict_instruct(self, config):
+        """Main entrypoint for executing the prediction compute plan.
 
-        : param config: configuration for the inference round
+        : param config: configuration for the prediction round
         """
+        # TODO: DEAD CODE?
+
         # Check/set instucting state
         if self.__state == ReducerState.instructing:
             logger.info("Already set in INSTRUCTING state")
@@ -455,20 +712,21 @@ class Control(ControlBase):
         # Set reducer in monitoring state
         self.__state = ReducerState.monitoring
 
-        # Start inference round
+        # Start prediction round
         try:
-            self.inference_round(config)
+            self.prediction_round(config)
         except TypeError:
             logger.error("Round failed.")
 
         # Set reducer in idle state
         self.__state = ReducerState.idle
 
-    def inference_round(self, config):
-        """Execute an inference round.
+    def prediction_round(self, config):
+        """Execute a prediction round.
 
-        : param config: configuration for the inference round
+        : param config: configuration for the prediction round
         """
+        # TODO: DEAD CODE?
         # Init meta
         round_data = {}
 
@@ -479,8 +737,8 @@ class Control(ControlBase):
 
         # Setup combiner configuration
         combiner_config = copy.deepcopy(config)
-        combiner_config["model_id"] = self.statestore.get_latest_model()
-        combiner_config["task"] = "inference"
+        combiner_config["model_id"] = self.db.model_store.get_active()
+        combiner_config["task"] = "prediction"
         combiner_config["helper_type"] = self.statestore.get_framework()
 
         # Select combiners
@@ -494,12 +752,12 @@ class Control(ControlBase):
             logger.warning("Round start policy not met, skipping round!")
             return None
 
-        # Synch combiners with latest model and trigger inference
+        # Synch combiners with latest model and trigger prediction
         for combiner, combiner_config in validating_combiners:
             try:
                 combiner.submit(combiner_config)
             except CombinerUnavailableError:
-                # It is OK if inference fails for a combiner
+                # It is OK if prediction fails for a combiner
                 self._handle_unavailable_combiner(combiner)
                 pass
 
