@@ -1,15 +1,18 @@
 import os
-import uuid
 from abc import ABC, abstractmethod
-from time import sleep
+from typing import Any, Dict, List, Tuple
 
 import fedn.utils.helpers.helpers
 from fedn.common.log_config import logger
 from fedn.network.api.network import Network
-from fedn.network.combiner.interfaces import CombinerUnavailableError
+from fedn.network.combiner.interfaces import CombinerInterface, CombinerUnavailableError
 from fedn.network.combiner.roundhandler import RoundConfig
 from fedn.network.state import ReducerState
+from fedn.network.storage.dbconnection import DatabaseConnection
 from fedn.network.storage.s3.repository import Repository
+from fedn.network.storage.statestore.stores.dto import ModelDTO
+from fedn.network.storage.statestore.stores.dto.round import RoundDTO
+from fedn.network.storage.statestore.stores.shared import SortOrder
 
 # Maximum number of tries to connect to statestore and retrieve storage configuration
 MAX_TRIES_BACKEND = os.getenv("MAX_TRIES_BACKEND", 10)
@@ -35,44 +38,25 @@ class ControlBase(ABC):
     :type statestore: :class:`fedn.network.statestore.statestorebase.StateStoreBase`
     """
 
+    repository: Repository
+
     @abstractmethod
-    def __init__(self, statestore):
+    def __init__(
+        self,
+        network_id: str,
+        repository: Repository,
+        db: DatabaseConnection,
+    ):
         """Constructor."""
         self._state = ReducerState.setup
 
-        self.statestore = statestore
-        if self.statestore.is_inited():
-            self.network = Network(self, statestore)
+        self.network = Network(self, network_id, db)
 
-        try:
-            not_ready = True
-            tries = 0
-            while not_ready:
-                storage_config = self.statestore.get_storage_backend()
-                if storage_config:
-                    not_ready = False
-                else:
-                    logger.warning("Storage backend not configured, waiting...")
-                    sleep(5)
-                    tries += 1
-                    if tries > MAX_TRIES_BACKEND:
-                        raise Exception
-        except Exception:
-            logger.error("Failed to retrive storage configuration, exiting.")
-            raise MisconfiguredStorageBackend()
+        self.repository = repository
 
-        if storage_config["storage_type"] == "S3":
-            self.model_repository = Repository(storage_config["storage_config"])
-        else:
-            logger.error("Unsupported storage backend, exiting.")
-            raise UnsupportedStorageBackend()
+        self.db = db
 
-        if self.statestore.is_inited():
-            self._state = ReducerState.idle
-
-    @abstractmethod
-    def session(self, config):
-        pass
+        self._state = ReducerState.idle
 
     @abstractmethod
     def round(self, config, round_number):
@@ -88,7 +72,14 @@ class ControlBase(ABC):
         :return: Helper instance.
         :rtype: :class:`fedn.utils.plugins.helperbase.HelperBase`
         """
-        helper_type = self.statestore.get_helper()
+        helper_type: str = None
+
+        try:
+            active_package = self.db.package_store.get_active()
+            helper_type = active_package.helper
+        except Exception:
+            logger.error("Failed to get active helper")
+
         helper = fedn.utils.helpers.helpers.get_helper(helper_type)
         if not helper:
             raise MisconfiguredHelper("Unsupported helper type {}, please configure compute_package.helper !".format(helper_type))
@@ -113,32 +104,15 @@ class ControlBase(ABC):
         else:
             return False
 
-    def get_model_info(self):
-        """:return:"""
-        return self.statestore.get_model_trail()
-
-    # TODO: remove use statestore.get_events() instead
-    def get_events(self):
-        """:return:"""
-        return self.statestore.get_events()
-
-    def get_latest_round_id(self):
-        last_round = self.statestore.get_latest_round()
-        if not last_round:
-            return 0
-        else:
-            return last_round["round_id"]
-
-    def get_latest_round(self):
-        round = self.statestore.get_latest_round()
-        return round
+    def get_latest_round_id(self) -> int:
+        return self.db.round_store.get_latest_round_id()
 
     def get_compute_package_name(self):
         """:return:"""
-        definition = self.statestore.get_compute_package()
+        definition = self.db.package_store.get_active()
         if definition:
             try:
-                package_name = definition["storage_file_name"]
+                package_name = definition.storage_file_name
                 return package_name
             except (IndexError, KeyError):
                 logger.error("No context filename set for compute context definition")
@@ -148,7 +122,7 @@ class ControlBase(ABC):
 
     def set_compute_package(self, filename, path):
         """Persist the configuration for the compute package."""
-        self.model_repository.set_compute_package(filename, path)
+        self.repository.set_compute_package(filename, path)
 
     def get_compute_package(self, compute_package=""):
         """:param compute_package:
@@ -157,23 +131,11 @@ class ControlBase(ABC):
         if compute_package == "":
             compute_package = self.get_compute_package_name()
         if compute_package:
-            return self.model_repository.get_compute_package(compute_package)
+            return self.repository.get_compute_package(compute_package)
         else:
             return None
 
-    def create_session(self, config: RoundConfig, status: str = "Initialized") -> None:
-        """Initialize a new session in backend db."""
-        if "session_id" not in config.keys():
-            session_id = uuid.uuid4()
-            config["session_id"] = str(session_id)
-        else:
-            session_id = config["session_id"]
-
-        self.statestore.create_session(id=session_id)
-        self.statestore.set_session_config(session_id, config)
-        self.statestore.set_session_status(session_id, status)
-
-    def set_session_status(self, session_id, status):
+    def set_session_status(self, session_id: str, status: str) -> Tuple[bool, Any]:
         """Set the round round stats.
 
         :param round_id: The round unique identifier
@@ -181,9 +143,11 @@ class ControlBase(ABC):
         :param status: The status
         :type status: str
         """
-        self.statestore.set_session_status(session_id, status)
+        session = self.db.session_store.get(session_id)
+        session.status = status
+        self.db.session_store.update(session)
 
-    def get_session_status(self, session_id):
+    def get_session_status(self, session_id: str):
         """Get the status of a session.
 
         :param session_id: The session unique identifier
@@ -191,23 +155,28 @@ class ControlBase(ABC):
         :return: The status
         :rtype: str
         """
-        return self.statestore.get_session_status(session_id)
+        session = self.db.session_store.get(session_id)
+        return session.status
 
-    def set_session_config(self, session_id: str, config: dict):
-        """Set the model id for a session.
+    def set_session_config(self, session_id: str, config: dict) -> Tuple[bool, Any]:
+        """Set the model id for a session
 
         :param session_id: The session unique identifier
         :type session_id: str
         :param config: The session config
         :type config: dict
         """
-        self.statestore.set_session_config_v2(session_id, config)
+        session = self.db.session_store.get(session_id)
+        session.session_config.patch_with(config)
+
+        self.db.session_store.update(session)
 
     def create_round(self, round_data):
         """Initialize a new round in backend db."""
-        self.statestore.create_round(round_data)
+        round = RoundDTO(**round_data)
+        self.db.round_store.add(round)
 
-    def set_round_data(self, round_id, round_data):
+    def set_round_data(self, round_id: str, round_data: dict):
         """Set round data.
 
         :param round_id: The round unique identifier
@@ -215,9 +184,11 @@ class ControlBase(ABC):
         :param round_data: The status
         :type status: dict
         """
-        self.statestore.set_round_data(round_id, round_data)
+        round = self.db.round_store.get(round_id)
+        round.round_data = round_data
+        self.db.round_store.update(round)
 
-    def set_round_status(self, round_id, status):
+    def set_round_status(self, round_id: str, status: str):
         """Set the round round stats.
 
         :param round_id: The round unique identifier
@@ -225,9 +196,11 @@ class ControlBase(ABC):
         :param status: The status
         :type status: str
         """
-        self.statestore.set_round_status(round_id, status)
+        round = self.db.round_store.get(round_id)
+        round.status = status
+        self.db.round_store.update(round)
 
-    def set_round_config(self, round_id, round_config: RoundConfig):
+    def set_round_config(self, round_id: str, round_config: RoundConfig):
         """Upate round in backend db.
 
         :param round_id: The round unique identifier
@@ -235,7 +208,9 @@ class ControlBase(ABC):
         :param round_config: The round configuration
         :type round_config: dict
         """
-        self.statestore.set_round_config(round_id, round_config)
+        round = self.db.round_store.get(round_id)
+        round.round_config = round_config
+        self.db.round_store.update(round)
 
     def request_model_updates(self, combiners):
         """Ask Combiner server to produce a model update.
@@ -249,7 +224,7 @@ class ControlBase(ABC):
             cl.append((combiner, response))
         return cl
 
-    def commit(self, model_id, model=None, session_id=None):
+    def commit(self, model: dict = None, session_id: str = None, name: str = None) -> str:
         """Commit a model to the global model trail. The model commited becomes the lastest consensus model.
 
         :param model_id: Unique identifier for the model to commit.
@@ -261,16 +236,38 @@ class ControlBase(ABC):
         """
         helper = self.get_helper()
         if model is not None:
-            logger.info("Saving model file temporarily to disk...")
             outfile_name = helper.save(model)
-            logger.info("CONTROL: Uploading model to Minio...")
-            model_id = self.model_repository.set_model(outfile_name, is_file=True)
+            logger.info("Saving model file temporarily to {}".format(outfile_name))
+            logger.info("CONTROL: Uploading model to object store...")
+            model_id = self.repository.set_model(outfile_name, is_file=True)
 
             logger.info("CONTROL: Deleting temporary model file...")
             os.unlink(outfile_name)
 
         logger.info("Committing model {} to global model trail in statestore...".format(model_id))
-        self.statestore.set_latest_model(model_id, session_id)
+
+        parent_model = None
+        if session_id:
+            last_model_of_session = self.db.model_store.list(1, 0, "committed_at", SortOrder.DESCENDING, session_id=session_id)
+            if len(last_model_of_session) == 1:
+                parent_model = last_model_of_session[0].model_id
+            else:
+                session = self.db.session_store.get(session_id)
+                parent_model = session.seed_model_id
+
+        new_model = ModelDTO()
+        new_model.model_id = model_id
+        new_model.parent_model = parent_model
+        new_model.session_id = session_id
+        new_model.name = name
+
+        try:
+            self.db.model_store.add(new_model)
+        except Exception as e:
+            logger.error("Failed to commit model to global model trail: {}".format(e))
+            raise Exception("Failed to commit model to global model trail")
+
+        return model_id
 
     def get_combiner(self, name):
         for combiner in self.network.get_combiners():
@@ -278,7 +275,7 @@ class ControlBase(ABC):
                 return combiner
         return None
 
-    def get_participating_combiners(self, combiner_round_config):
+    def get_participating_combiners(self, combiner_round_config) -> List[Tuple[CombinerInterface, Dict]]:
         """Assemble a list of combiners able to participate in a round as
         descibed by combiner_round_config.
         """
