@@ -1,4 +1,5 @@
 from concurrent import futures
+from typing import TypedDict
 
 import grpc
 from grpc_health.v1 import health, health_pb2_grpc
@@ -8,14 +9,42 @@ from fedn.common.log_config import logger, set_log_level_from_string, set_log_st
 from fedn.network.grpc.auth import JWTInterceptor
 
 
+class ServerConfig(TypedDict):
+    port: int
+    secure: bool
+    key: str
+    certificate: str
+    logfile: str
+    verbosity: str
+
+
 class Server:
     """Class for configuring and launching the gRPC server."""
 
-    def __init__(self, servicer, modelservicer, config):
+    def __init__(self, servicer, modelservicer, config: ServerConfig):
         set_log_level_from_string(config.get("verbosity", "INFO"))
         set_log_stream(config.get("logfile", None))
 
-        self.server = grpc.server(futures.ThreadPoolExecutor(max_workers=350), interceptors=[JWTInterceptor()])
+        # Keepalive settings: these detect if the client is alive
+        KEEPALIVE_TIME_MS = 60 * 1000  # send keepalive ping every 60 second
+        # wait 30 seconds for keepalive ping ack before considering connection dead
+        KEEPALIVE_TIMEOUT_MS = 30 * 1000
+        # max idle time before server terminates the connection (5 minutes)
+        MAX_CONNECTION_IDLE_MS = 5 * 60 * 1000
+
+        MAX_PING_STRIKES = 2
+
+        self.server = grpc.server(
+            futures.ThreadPoolExecutor(max_workers=350),
+            interceptors=[JWTInterceptor()],
+            options=[
+                ("grpc.keepalive_time_ms", KEEPALIVE_TIME_MS),
+                ("grpc.keepalive_timeout_ms", KEEPALIVE_TIMEOUT_MS),
+                ("grpc.max_connection_idle_ms", MAX_CONNECTION_IDLE_MS),
+                ("grpc.http2.max_pings_without_data", 0),  # Allow unlimited PINGs without data
+                ("grpc.http2.max_ping_strikes", MAX_PING_STRIKES),
+            ],
+        )
         self.certificate = None
         self.health_servicer = health.HealthServicer()
 
